#Read a configuration file.
#This file is mandatory and must be created by hand.
proc romanicReadConfFile {} {
    global gvar deviceType robotType writerAddressList

	if {![file exists ~/.romanicConfig]} {
	puts "Configuration file is missing."
	puts "Please create a file under you home directory named .romanicConfig"
	puts "and containing entries in the format:"
	puts "      <device address><TAB><tty><TAB><baud rate><TAB><writer type><TAB><robot type><TAB><parallel port><TAB><robot mode>"
	puts "Eg."
	puts "0,0,0\t/dev/tt1p0\t38400\tCD\tRIMAGE\t/dev/lpr1\tmixedMode\t1"
	puts "0,1,0\t/dev/tt1p0\t38400\tDVD\tRIMAGE\t/dev/lpr1\tmixedMode\t2"
	puts "1,3,0\t/dev/tt0p0\t115200\tCD\tLSK\t/dev/lpr1\tthreeMagazineMode\t1"
	puts "1,5,0\t/dev/tt0p0\t115200\tDVD\tLSK\t/dev/lpr1\tthreeMagazineMode\t2"
	exit 1
    }

    if {![file readable ~/.romanicConfig]} {
	puts "Configuration file ~/.romanicConfig exists but is not readable"
	exit 1
    }

    if {[catch {set conFile [open ~/.romanicConfig r]}]} { 
	puts "Cannot open ~/.romanicConfig"
	exit 1
    }

####################################################################
    while {[gets $conFile line] > 0} {
#puts "Reading line: $line"	
	if {[llength $line] != 8} {
	    puts "Configuration file corrupted."
	    puts "Exit."
	    exit 1
	}
	
	set device  [lindex $line 0]
	set tty     [lindex $line 1]
	set baud    [lindex $line 2]
	set type    [string toupper [lindex $line 3]]
	set robType [string toupper [lindex $line 4]]
	set lpt     [lindex $line 5]
	set robMode [string tolower [lindex $line 6]]
	set writerAddress [lindex $line 7]

	if {[checkBaud $baud] == "error"} {
	    puts "baud rate of $baud is invalid."
	    puts "Check ~/.romanicConfig"
	    puts "Exit."
  	    exit 1
	}

	set validateRobot [checkRobotType $robType]
	if {$validateRobot != "Ok"} {
	    puts "robot type $robType is invalid."
 	    puts "valid robot types are: $validateRobot"
	    puts "Check ~/.romanicConfig"
	    puts "Exit."
  	    exit 1
	}


	if {![file exists $lpt] || ![file writable $lpt]} {
	    puts "Parallel port $lpt doesn't exist or is not readable."
	    puts "Check ~/.romanicConfig"
	    puts "Exit."
  	    exit 1
	}

	set validateRobotMode [checkRobotMode $robMode]
	if {$validateRobotMode != "Ok"} {
	    puts "robot mode $robMode is invalid."
 	    puts "valid robot types are: $validateRobotMode"
	    puts "Check ~/.romanicConfig"
	    puts "Exit."
  	    exit 1
	}

	lappend confList "$device $tty $baud $type $robType $lpt $robMode $writerAddress"
    }

    set ttyList     ""
    set deviceList()  ""
    set deviceType()  ""
    set deviceSpeed() ""
    set robotType()   ""
    set robotMode()   ""
    set parallelList()  ""
    set writerAddressList() ""
    foreach confLine $confList {
	set device  [lindex $confLine 0]
	set tty     [lindex $confLine 1]
	set baud    [lindex $confLine 2]
	set type    [lindex $confLine 3]
	set robType [lindex $confLine 4]
	set lpt     [lindex $confLine 5]
	set robMode [lindex $confLine 6]
	set robotWriter  [lindex $confLine 7]
	if {[lsearch -exact $ttyList $tty] == -1} {
	    lappend ttyList $tty
	    lappend deviceList($tty)    $device
	    lappend deviceSpeed($tty)   $baud
	    lappend deviceType($device) $type
	    lappend robotType($tty)     $robType
	    lappend parallelList($tty)  $lpt
	    lappend robotMode($tty)     $robMode
	    lappend writerAddressList($device) $robotWriter
	} else {
	    if {[lsearch -exact deviceList($tty) $device] == -1} {
		lappend deviceList($tty)    $device
		lappend deviceSpeed($tty)   $baud
		lappend deviceType($device) $type
		lappend writerAddressList($device)  $robotWriter
	    } else { Debug "Duplicate line in config file. Skip." 1 }
	}
    }
    
    foreach tty $ttyList {
	lappend gvar(ttyList)      $tty
	lappend gvar(deviceList)   $deviceList($tty)
	lappend gvar(speedList)    $deviceSpeed($tty)
	lappend gvar(robotList)    $robotType($tty)
	lappend gvar(parallelList) $parallelList($tty)
	lappend gvar(robotMode)    $robotMode($tty)
	foreach device $deviceList($tty) {
	    #puts "Associating to gvar($device) the tty $tty"
	    set gvar($device) $tty
	}	

	forkProcess "romanicProcInitializeLoop"
	after 1000
	set gvar(ttyList)    ""
	set gvar(deviceList) ""
	set gvar(speedList)  ""
	set gvar(robotList)  ""
	set gvar(parallelList) ""
	set gvar(robotMode)    ""
    }
}

#Reset all the (possible) robot(s) connected.
proc romanicResetRobotAll {} {
    global gvar
    
    foreach tty $gvar(ttyList) {
	Log "Reset robot connected to $tty"
	romanicResetRobot $tty
    }
}

#Reset a robot connected to a specific port.
proc romanicResetRobot {tty} {
    global gvar

    set gvar(tty) $tty
    
    puts "Close printer - $tty"
    Log "Close printer - $tty"
    closePrinter
    
    puts "Close all the writers"
    Log "Close all the writers"
    closeWriters 
    
    puts "Free the arm in case there are media stuck in it."
    Log "Free the arm in case there are media stuck in it."
    freeArm $tty

    puts "Check if there is a medium in the printer - $tty"
    Log "Check if there is a medium in the printer - $tty"
    checkPrinter $tty

    puts "Check if there are media in the writers - $tty"
    Log "Check if there are media in the writers - $tty"
    checkWriters $tty

    armOnTop
    fanOff
}

#Calculate the number of CDs in all magazines.
proc romanicCalculateNumCDsAll {} {
    global gvar 

    puts "Calculate number of media in the magazine."
    Log "Calculate number of CDs in the magazine."
    foreach tty $gvar(ttyList) {
	romanicCalculateNumCDs $tty
    }
}


proc romanicCalculateNumCDs {tty} {
    global gvar
    
    set gvar(numBlankCDs)   [checkMagazine $tty blank]
    set gvar(numWrittenCDs) [checkMagazine $tty written]
    set gvar(numBadCDs)     [checkMagazine $tty bad]
    set ttyTail             [file tail $tty]
    
    Debug "Media in Blank magazine:   $gvar(numBlankCDs)" 1
    Debug "Media in Written magazine: $gvar(numWrittenCDs)" 1
    Debug "Media in Bad magazine:     $gvar(numBadCDs)" 1


    if {$gvar(numBlankCDs) == "error"} {	
	puts "Error reading number of media in the Blank magazine"
	puts "In robot $ttyTail"
	exit 1
    }
    
    if {$gvar(numWrittenCDs) == "error"} {	
	puts "Error reading number of media in the Written magazine"
	puts "In robot $ttyTail"
	exit 1
    }
    
    if {$gvar(numBadCDs) == "error"} {	
	puts "Error reading number of media in the Bad magazine"
	puts "In robot $ttyTail"
	exit 1
    }
    

    if {$gvar(numBlankCDs) > 60} {
	puts "The magazine n. 1 contains $gvar(numBlankCDs)"
	puts "Maximum allowed is 60 media"
	puts "In robot $ttyTail"
	exit 1
    }

    if {$gvar(robotMode) == "3-magazine"} {
	
	if {$gvar(numBlankCDs) == 0} {
	    puts "Magazine number 1 is empty."
	    puts "In robot $ttyTail"
	    exit 1
	}
	
	if {[expr $gvar(numBlankCDs) + $gvar(numWrittenCDs) +$gvar(numBadCDs)]  > 60} {
	    puts "There are too many media in the carousel."
	    puts "Please empty the magazine 1 and/or 2."
	    puts "In robot $ttyTail"
	    exit 1
	}
	
    } elseif {$gvar(robotMode) == "efficient"} {	
	
	if {$gvar(numBlankCDs) == 0 && $gvar(numWrittenCDs) == 0} {
	    puts "Magazines 1 and 2 are empty."
	    puts "In robot $ttyTail"
	    exit 1
	}
	
	if {$gvar(numWrittenCDs) > 60} {
	    puts "The magazine n. 2 contains $gvar(numWrittenCDs)"
	    puts "Maximum allowed is 60 media"
	    puts "In robot $ttyTail"
	    exit 1
	}
    
	if {[expr $gvar(numBlankCDs) + $gvar(numBadCDs)]  > 60} {
	    puts "There are too many media in the carousel."
	    puts "Please empty the magazine 3."
	    puts "In robot $ttyTail"
	    exit 1
	}	
    
	if {$gvar(numBlankCDs) == 0} { 
	    set gvar(inputMagazine)  2
	    set gvar(outputMagazine) 1
	} else {
	    set gvar(inputMagazine)  1
	    set gvar(outputMagazine) 3
	}
	
	puts "Write $gvar(reqDir)/currentMagazine-$gvar(tty).romanic"
	puts "Input magazine is: $gvar(inputMagazine)"
	puts "Output magazine is: $gvar(outputMagazine)"
	write_file $gvar(reqDir)/currentMagazine-$ttyTail.romanic $gvar(inputMagazine)
    
    } else {

	if {[expr $gvar(numBlankCDs) + $gvar(numBadCDs)]  > 60} {
	    puts "There are too many media in the carousel."
	    puts "Magazine 1 must have no more than 30 CD/DVDs"
	    puts "and magazine 3 must be empty."
	    puts "In robot $ttyTail"
	    exit 1
	}
	
	if {[expr $gvar(numWrittenCDs) + $gvar(numBadCDs)]  > 60} {
	    puts "There are too many media in the carousel."
	    puts "Magazine 2 must have no more than 30 CD/DVDs"
	    puts "and magazine 3 must be empty."
	    puts "In robot $ttyTail"
	    exit 1
	}
	
	if {[expr $gvar(numWrittenCDs) + $gvar(numBlankCDs)]  > 60} {
	    puts "There are too many media in the carousel."
	    puts "Magazine 2 must have no more than 30 CD/DVDs"
	    puts "and magazine 3 must be empty."
	    puts "In robot $ttyTail"
	    exit 1
	}
    }
}




#Count the CDs of all the magazines.
proc checkAll {tty} {

    foreach magazine {blank written bad} {
	checkMagazine $tty $magazine	
    }
}


#Check if there are request files. If there are already 2 requests
#queue, return: each robot has 2 writing devices only.
#I know, I can print while the writers are busy, but let's not
#complicate to much the machinery...
proc globLoop {} {
    global gvar 
    
    set tty [file tail $gvar(ttyList)]
    set requestsPending [glob -nocomplain $gvar(reqDir)/inProcess-*-$tty.romanic]
    if {[llength $requestsPending] >= 2} {
	return
    }
    
    set requests [glob -nocomplain $gvar(reqDir)/request-*.romanic]
   
    if {$requests == ""} { 
	return 
    } 
    
    #Check if a request is already in process. If not, queue it.
    foreach request $requests {
	set id [string range [file tail [file root $request]] 8 end]
	if {[file exists $gvar(reqDir)/inProcess-$id-$tty.romanic]} { 
	    Debug "id: $id is already in queue" 3
	    continue
	} else {
	    if {[catch {set file [open $request r]} msg]} {
		continue
	    }
	    gets $file requestLine
	    close $file
	    if {[checkDevice $requestLine]} { 
		continue
	    } else {
		lappend gvar(requests) "-id $id $requestLine"
	    }
	}
    }
    return 
}

#Serve all the requests in the queue.
proc serveRequestsLoop {} {
    global gvar

    if {$gvar(requests) == ""} { 
	#No requests pending. Goodbye.
	return 
    }
        
    set tty [file tail $gvar(ttyList)]
    #Process the request if not doing it already.
    foreach request $gvar(requests) {
	set id [setId $request]
	if {[file exists $gvar(reqDir)/inProcess-$id-$tty.romanic]} { 
	    Debug "Already processing request $id" 3
	    continue 
	} else {
	    write_file $gvar(reqDir)/inProcess-$id-$tty.romanic ""
	    set index [lsearch  $gvar(requests) "*-id $id*"]
	    set gvar(requests) [lreplace $gvar(requests) $index $index]
	    switch -- $gvar(robotMode) {
		"3-magazine" {forkProcess "threeMagazineMode \"$request\""}
		"efficient"  {forkProcess "efficientMode \"$request\""}
		"mixed"      {forkProcess "mixedMode \"$request\""}
		default { return } 
	    } 	
	}
    }
}


#Extract the id from the request.
proc setId {request} {

    set idIndex [lsearch -exact $request "-id"]
    if {$idIndex == -1} { 
	return "error"
    } else {
	set id [lindex $request [incr idIndex]]
	return $id
    }
}

#Check if this request belongs to the current tty-process.
proc checkDevice {requestLine} {
    global gvar

    if {[lindex $requestLine 0] == "printOnly"} { return 1 }
    
    set device [lindex $requestLine [expr [lsearch -exact $requestLine "-writerAddress"] +1]]
    
    #This request is not for this process...
    if {![info exists gvar($device)]} { 
	#puts "The device $gvar($device) is not in the config file."
	Debug "The device gvar($device) is not in the config file." 3
	return 1 
    }

    if {[lindex $gvar($device) 0] != $gvar(ttyList)} { 
	#puts "The device $gvar($device) is not in the device list"
	Debug "The device $gvar($device) is not in the device list" 3
	return 1
    }

    if {[getBookedDevice $device]} {
	#puts "The device $gvar($device) is currently in use"
	Debug "The device $gvar($device) is currently in use" 3
	return 1 
    } else {
	bookDevice $device
    }
    
    return 0
}

proc getBookedDevice {device} {
    global gvar 

    if {[file exists $gvar(reqDir)/writerLock-$device.romanic]} { 
	Debug "Writer $device is busy" 3
	return 1
    } else { 
	return 0 
    }
}

proc bookDevice {device} {
    global gvar

    if {[catch {write_file $gvar(reqDir)/writerLock-$device.romanic ""} msg]} {
	Debug "I cannot create the lock file $gvar(reqDir)/writerLock-$device.romanic because: $msg" 1
	waitForFixError "I cannot create the lock file $gvar(reqDir)/writerLock-$device.romanic"
    }    
}

proc unLockDevice {device} {
    global gvar

    if {[catch {file delete $gvar(reqDir)/writerLock-$device.romanic ""} \
	     msg]} {
	Debug "I cannot remove the lock file $gvar(reqDir)/writerLock-$device.romanic" 1
	waitForFixError "I cannot remove the lock file $gvar(reqDir)/writerLock-$device.romanic"
    }
}

#Check if the request is valid.
proc checkRequest {request} {
    global gvar deviceType

    set id [setId $request]
    
    if {$id == "error"} { 
	writeError "Error: id $id  not valid"
	return 1
    } else {
	set gvar(id) $id
    }


    #Parse all the fields of the request:
    #-action= action to perfom. It can be: 
    #         writePrint: write a cd and print it.
    #         writeOnly:  just write a CD.
    #         printOnly:  just print a CD.
    #-volumeFile= path and name of the volume file to write.
    #-volumeName= name of the volume to use to write a CD.
    #-writeAddress= address of the writer in cdrecord format (Eg. 0,1,0)
    #-writerSpeed= speed of the writer
    #-dummymode= don't really write a CD.
    #-image= path and name of the image to print.

    for { set i 0} {$i <[llength $request]} {incr i} {
     
        switch -- [lindex $request $i] {
	    -id {incr i}
	    -action {
		incr i
		set gvar($id-action) [lindex $request $i]
	    }
	    -volumeFile {
		incr i
		set gvar($id-volumeFile) [lindex $request $i]
	    } 
	    -volumeName {
		incr i
		set gvar($id-volumeName) [lindex $request $i]
	    } 
	    -copies {
		incr i
		set gvar($id-copies) [lindex $request $i]
	    } 
	    -writerAddress {
		incr i
		set gvar($id-writerAddress) [lindex $request $i]
	    } 
	    -writerSpeed {
		incr i
		set gvar($id-writerSpeed) [lindex $request $i]
	    } 
	    -dummyMode {
		 set gvar($id-dummyMode) 1
	    }
	    -image {
		incr i
		set gvar($id-imageToPrint) [lindex $request $i]
	    }
	    -medium {
		incr i
		set gvar($id-medium) [lindex $request $i]
	    }
	    default {
		Debug "Unknown option: [lindex $request $i]" 1
		return 1
	    }
	}
    }
    
    switch $gvar($id-action) {
	"writePrint" {set gvar($id-print) 1 ; set gvar($id-write) 1}
	"writeOnly"  {set gvar($id-print) 0 ; set gvar($id-write) 1}
	"printOnly"  {set gvar($id-print) 1 ; set gvar($id-write) 0}
	default      {Debug "Unknown option $gvar($id-action)" 1 ; return 1 }
    }
    
    if {$gvar($id-action) == "printOnly"} {
	if {![info exists gvar($id-volumeFile)] || $gvar($id-volumeFile) == ""} {
	    writeError "volume file not specified."
	    return 1
	}
    }
	
	if {![info exists gvar($id-writerAddress)] || $gvar($id-writerAddress) == ""} {
	    writeError "writer address not specified."
	    return 1
	}
	
	if {![info exists gvar($id-medium)] || $gvar($id-medium) == ""} {
	    writeError "medium type not specified"
	    return 1
	}
	
	if {$gvar($id-medium) != $deviceType($gvar($id-writerAddress)) && \
		$deviceType($gvar($id-writerAddress)) != "CD/DVD"} {
	    writeError "medium type $gvar($id-medium) is wrong (check .romanicConfig)"
	    return 1
	}

	if {![info exists gvar($id-writerSpeed)] || $gvar($id-writerSpeed) == ""} {
	    set gvar($id-writerSpeed) 2
	}
	
	if {![info exists gvar($id-volumeName)] || $gvar($id-volumeName) == ""} {
	    set gvar($id-volumeName) [file root [file tail $gvar($id-volumeFile)]]
	}
    

    if {![info exists gvar($id-copies)] || $gvar($id-copies) == ""} {
	set gvar($id-copies) 1
    }
    
    if {![info exists gvar($id-dummyMode)]} {
	set gvar($id-dummyMode) 0
    }

    if {![info exists gvar($id-writerSpeed)]} {
	set gvar($id-writerSpeed) 1
    }

    return 0
}

#Remove an entry from the queue list.
proc cleanUpQueue {id} {
    global gvar 

    Log "Cleanup queues"
    set tty [file tail $gvar(ttyList)]

    set index [lsearch $gvar(requests) "*-id $id*"]
    if {$index != -1} {
	set gvar(requests) [lreplace $gvar(requests) $index $index]
    }
    
    if {[info exists gvar($id-writerAddress)]} {
	file delete $gvar(reqDir)/writerLock-$gvar($id-writerAddress).romanic
    } 
    file delete $gvar(reqDir)/request-$id.romanic
    file delete $gvar(reqDir)/inProcess-$id-$tty.romanic
    file delete $gvar(reqDir)/count-$id.romanic
    unsetId $id
}

#Remove temporary files.
proc cleanUpAll {} {
    global gvar

    set inProcessFiles [glob -nocomplain $gvar(reqDir)/inProcess-*]
    set lockFiles      [glob -nocomplain $gvar(reqDir)/*.lock]
    set errorFiles     [glob -nocomplain $gvar(reqDir)/error-*.romanic]
    set fixFiles       [glob -nocomplain $gvar(reqDir)/fixed-*.romanic]
    set countFiles     [glob -nocomplain $gvar(reqDir)/count-*.romanic]
    set fatalErrors    [glob -nocomplain $gvar(reqDir)/fatalError-*.romanic]
    set fanFiles       [glob -nocomplain $gvar(reqDir)/fan-*.romanic]
    set currentMag     [glob -nocomplain $gvar(reqDir)/currentMagazine-*.romanic]
    set cdLabels       [glob -nocomplain $gvar(reqDir)/cdLabel-*]
    set lockFiles      [glob -nocomplain $gvar(reqDir)/writerLock-*]
    
    foreach list "$inProcessFiles $lockFiles $errorFiles $fixFiles $countFiles $fatalErrors $fanFiles $currentMag $cdLabels" {
	foreach file $list {
	    Log "Delete file: $file"
	    file delete $file
	}
    }
}

#Unset some variables.
proc unsetId {id} {
    global gvar

    unset gvar(id)
    unset gvar($id-action)
    catch {unset gvar(tty)}
    catch {unset gvar(ttyTail)}
    catch {unset gvar($id-volumeFile)}
    catch {unset gvar($id-volumName) }
    catch {unset gvar($id-copies) }
    catch {unset gvar($id-writerAddress)}
    catch {unset gvar($id-writerSpeed) }
    catch {unset gvar($id-dummyMode) }
    catch {unset gvar($id-imageToPrint)}
    catch {unset gvar($id-write) }
    catch {unset gvar($id-print) }
}


##########################################
