#! /bin/sh
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2002 European Southern Observatory
# LICENSE
#
# PROJECT:   VLT Data Flow System
# AUTHOR:    Carlos Guira
# CI NUMBER:
# DESCRIPTION: Updates the /vltdata/ENVIRONMENTS/`hostname` to allow only 
#              one process to register itself as "schedule" 
# REMARKS:     It follows the syntax of other vccEnv* commands:
#
#                     vccEnvDfsSetup -e $RTAPENV
#
# $Revision: 1.2 $
#******************************************************

cmd=`basename $0`

#
# Checking echo
#
echo=echo
if [ "`echo -n`" = "-n" ] ; then
    SV_NONL="\c"
else
    echo="echo -n"
fi

#
# At least two parameters are required:  -e $RTAPENV
#
if [ $# -lt  2 ]; then
  echo "$cmd: Wrong number of parameters."
  echo "Usage: $cmd -e \$RTAPENV"
  exit 1
fi

if [ "$1" !=  "-e" ]; then
  echo "$cmd: Wrong parameter."
  echo "Usage: $cmd -e \$RTAPENV"
  exit 1
fi

RTAPENV=$2
CcsEnvTable=/vltdata/ENVIRONMENTS/${RTAPENV}/CcsEnvTable
#
# If file exist check first if the 'schedule' line is already added,
# otherwise add it to the end of the file
#
if [ -f "$CcsEnvTable" ] ; then
  echo "grep "schedule\$" $CcsEnvTable > /dev/null 2>&1"
  grep "schedule\$" $CcsEnvTable > /dev/null 2>&1
  if [ "$?" = "0" ]; then
    echo "$cmd: File $CcsEnvTable already contains an 'schedule' entry"
    exit 0
  else
    cat << EOF >> $CcsEnvTable
# Only one process should register itself as "schedule", "p2pp", "ot" or "masktracker" 
# - column 5 must be "N"
    0  0  N  N  N  Y  N    1  128   schedule
    0  0  N  N  N  Y  N    1  128   p2pp
    0  0  N  N  N  Y  N    1  128   ot
    0  0  N  N  N  Y  N    1  128   masktracker
EOF
    echo "$cmd: Entry 'schedule' added to $CcsEnvTable"
    echo "$cmd: Entry 'p2pp' added to $CcsEnvTable"
    echo "$cmd: Entry 'ot' added to $CcsEnvTable"
    exit 0
  fi
else
  echo "$cmd: File $CcsEnvTable no such file"
  exit 1
fi
