package org.eso.dfs.datatransfer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.List;

/**
 * @author nkornwei
 *
 * Implementation of the Transferable interface used for communicating a
 * file list (List object containing File objects).
 */
public class FileListTransferable implements Transferable {

    /**
     * Index of fileList flavour in the flavors array.
     */
    private static final int FileListFlavorIndex = 0;
    
    /**
     * Array of flavors supported by this transferable.
     */
    private static final DataFlavor[] flavors = 
        { DataFlavor.javaFileListFlavor };

    /**
     * The data being transfered.
     */
    private List fileList;
    
    /**
     * Constructor for FileListTransferable.
     */
    public FileListTransferable(List fList) {
        super();
        fileList = fList;
    }

    /**
     * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
     */
    public DataFlavor[] getTransferDataFlavors() {
        return (DataFlavor[]) flavors.clone();
    }

    /**
     * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(java.awt.datatransfer.DataFlavor)
     */
    public boolean isDataFlavorSupported(DataFlavor flavor) {
        for (int i = 0; i < flavors.length; i++) {
            if (flavor.equals(flavors[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * @see java.awt.datatransfer.Transferable#getTransferData(java.awt.datatransfer.DataFlavor)
     */
    public Object getTransferData(DataFlavor flavor)
        throws UnsupportedFlavorException, IOException {
        {
            if (flavor.equals(flavors[FileListFlavorIndex])) {
                return (Object) fileList;
            } else {
                throw new UnsupportedFlavorException(flavor);
            }
        }

    }
}
