/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */

package org.eso.dfslog;

import java.util.*;
import java.text.*;
import java.io.*;

// JDBC
import java.sql.*;


/**
 * ConnServerSQLReport:
 *
 * This class extends the Report class, generating a report
 * from the ResultSet of an SQL query. The constructor requires
 * a ConnectionServer object in order to execute the query.
 *
 */

public class ConnServerSQLReport extends Report {

    private String header;
    private PrintWriter pw = null;
    private String footer;
    private String sqlStatement;
    private Statement stmt = null;
    private ConnectionServer connSrvr = null;

    private int columnWidths[] = null;

    public ConnServerSQLReport(ConnectionServer cs, String s) {
        sqlStatement = s;
        connSrvr = cs;
        SimpleDateFormat formatter
            = new SimpleDateFormat("dd MMM yyyy hh:mm:ss");
        header = new String("\nReport Title: DFSLog Messages\n\n"
            +"Date Generated: "+formatter.format(new java.util.Date())+"\n");
        footer = new String("End of Report");
    }




    public void cleanup() {
        System.out.println("finished printing...");//NK
        try {
            if (stmt != null) {
                connSrvr.releaseStatement(stmt);
            }
        } catch (SQLException e) {
            System.out.println("Error closing Statement: "+e.toString());
        } finally {
            // set to null so it can not be released again.
            stmt = null;
        }
    }


    public int[] getColumnWidths() {
        return columnWidths;
    }
 
 
    public void setHeader(String s) {
        header = s;
    }
 
    public void setFooter(String s) {
        footer = s;
    }
 
    public String getReportHeader() {
        return header;
    }
 
    public String getReportFooter() {
        return footer;
    }




    private void storeWidth(String s, int c) {
        if (s != null && columnWidths != null
                && columnWidths[c] < s.length()) {
            columnWidths[c] = s.length();
        }
    }
 
 
 
 
    public void setPrintWriter(PrintWriter p) {
        pw = p;
    }




    public boolean generateReport() {
         
        if (pw == null) {
            return false;
        }
        boolean rc = true;
        pw.print(header);
        pw.println("Data Source: "+sqlStatement+"\n");
        ResultSet rs;
        ResultSetMetaData rsmd;
        try {
            // get a Statement - we will wait() until one is available
            // by using requestStatement() instead of getStatement().
            stmt = connSrvr.requestStatement();
            boolean results = stmt.execute(sqlStatement);
            if (!results) {
                pw.println("No data for this query");
            } else {
                do {
                    rs = stmt.getResultSet();
                    rsmd = rs.getMetaData();
                    StringBuffer column;
                    int numColumns = rsmd.getColumnCount();
                    if (columnWidths == null) {
                        columnWidths = new int[numColumns+1];
                        for (int i = 0; i < numColumns+1; i++) {
                            columnWidths[i] = 0;
                        }
                    }

                    for(int rowNum = 1; rs.next(); rowNum++) {
                        column = new StringBuffer("[ " + rowNum + "]");
                        storeWidth(column.toString(), 0);
                        for (int i = 1; i <= numColumns; i++) {
                            String field = rs.getString(i);
                            column.append("\t" + field);
                            storeWidth(field, i);
                        }
                        pw.println(column.toString());
                        if (isInterrupted()) {
                            break;
                        }
                        // yield this CPU intensive thread.
                        Thread.currentThread().yield();
                    }
                    if (isInterrupted()) {
                        break;
                    } else {
                        results = stmt.getMoreResults();
                    }
                } while (results);
            }
        } catch (SQLException e) {
            pw.println("Error executing SQL: "+e.toString());
        } finally {
            cleanup();
        }
        pw.print(footer);
        return rc;
    }

}  // end of class ConnServerSQLReport.
// end of file ConnServerSQLReport.java.
