/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;

import java.util.*;
import java.text.*;
import java.io.*;
import javax.swing.table.*;


/**
 * TableModelReport:
 *
 * An extension of the Report interface for generation of reports
 * from selected rows in a TableModel object.
 */

public class TableModelReport extends Report {

    private String header;
    private String footer;
    private TableModel model;
    private PrintWriter pw = null;

    private int columnWidths[] = null;
    private int startRow;
    private int nRows;

    public TableModelReport(TableModel m, int sr, int nr) {
        model = m;
        startRow = sr;
        nRows = nr;
        SimpleDateFormat formatter
            = new SimpleDateFormat("dd MMM yyyy hh:mm:ss");
        header = new String("\nReport Title: DFSLog Messages\n\n"
            +"Date Generated: "+formatter.format(new java.util.Date())+"\n");
        footer = new String("End of Report");
    }




    public int[] getColumnWidths() {
        return columnWidths;
    }

 
   

    public void setHeader(String s) {
        header = s;
    }

    public void setFooter(String s) {
        footer = s;
    }

    public String getReportHeader() {
        return header;
    }

    public String getReportFooter() {
        return footer;
    }




    public void setPrintWriter(PrintWriter p) {
        pw = p;
    }




    public boolean generateReport() {
        if (pw == null) {
            return false;
        }
        boolean rc = true;
        pw.print(header);
        int r = startRow;
        int cols = model.getColumnCount();
        columnWidths = new int[cols+1];
        for (int i = 0; i < cols+1; i++) {
            columnWidths[i] = 0;
        }
        pw.print("Data Source: DFSLog Report of user selected rows\n\n");
        while (r < startRow+nRows) {
            StringBuffer column;
            column = new StringBuffer("[ " + r + "]");
            storeWidth(column.toString(), 0);
            for (int c = 0; c < cols; c++) {
                String field = (String)model.getValueAt(r, c);
                storeWidth(field, c+1);
                column.append("\t"+(String)model.getValueAt(r, c));
            }
            // yield this CPU intensive thread.
            Thread.currentThread().yield();
            pw.println(column.toString());
            r++;
        }
        pw.println(footer);
        return rc;
    }



    public void cleanup() {
        return;
    }




    private void storeWidth(String s, int c) {
        if (columnWidths != null && columnWidths[c] < s.length()) {
            columnWidths[c] = s.length();
        }
    }

}  // end of class TableModelReport.
// end of file TableModelReport.java.
