package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;

import org.eso.gasgano.properties.GasProp;

/**
 * @author nkornwei
 *
 * Database property editing.
 */
public class DBPanel extends PropertyEditorSubPanel implements ActionListener {

    private GasProp gpi;

    private JButton insertRow;
    private JButton deleteRow;

    private JTable table;
    private JScrollPane scrollPane;
    private JCheckBox obsStatReq;
    private JCheckBox databaseReq;
    private JTextField obServerUrl;
    private JTextField obUserName;
    private JTextField dbServerUrl;
    private JTextField dbServerId;
    private JTextField dbUserName;

    private JPanel lowerPanel;

    private Vector obManagedVector = new Vector();
    private Vector dbManagedVector = new Vector();

    private ATModel tableModel;

    /**
     * Table model for OB Status values and labels.
     */
    class ATModel extends AbstractTableModel {

        private String[] columnNames = { "OB Status", "Label" };
        private GasProp gpi;

        /*
         * Constructor.
         */
        ATModel(GasProp gp) {
            loadProperties(gp);
        }

        void loadProperties(GasProp gp) {
            gpi = gp;
            // here we have to make sure that if there was 
            // no value in the vector we can still add new rows
            if (gpi.obsStatus.size() == 0) {
                insertRow(0);
            }
            fireTableDataChanged();
        }

        public int getColumnCount() {
            return 2;
        }
        public int getColumnWidth(int column) {
            if (column == 0) {
                return 170;
            } else {
                return 170;
            }
        }
        public String getColumnName(int column) {
            return columnNames[column];
        }
        public int getRowCount() {
            return gpi.obsStatus.size();
        }

        public Object getValueAt(int row, int col) {
            if (col == 0) {
                return gpi.obsStatus.elementAt(row).toString();
            } else {
                return gpi.obsStatusMeaning.elementAt(row).toString();
            }
        }
        public boolean isCellEditable(int row, int col) {
            return true;
        }
        public void setValueAt(Object aValue, int row, int column) {
            if (column == 0) {
                gpi.obsStatus.setElementAt((String) aValue, row);
            } else {
                gpi.obsStatusMeaning.setElementAt((String) aValue, row);
            }
        }

        public Class getColumnClass(int col) {
            return getValueAt(0, col).getClass();
        }

        public void removeRow(int rowNum) {
            try {
                table
                    .getDefaultEditor(table.getColumnClass(1))
                    .stopCellEditing();
                gpi.obsStatus.removeElementAt(rowNum);
                gpi.obsStatusMeaning.removeElementAt(rowNum);
                fireTableDataChanged();
            } catch (ArrayIndexOutOfBoundsException e) {
                if (isDebug()) {
                    System.out.println("GasPropEditor: array out of bound");
                }
            }
        }
        public void insertRow(int rowNum) {
            try {
                gpi.obsStatus.insertElementAt("", rowNum);
                gpi.obsStatusMeaning.insertElementAt("", rowNum);
                fireTableDataChanged();
            } catch (ArrayIndexOutOfBoundsException e) {
                if (isDebug()) {
                    System.out.println("GasPropEditor: array out of bound");
                }
            }
        }
    }

    public DBPanel(GasProp gp) {

        JPanel buttonsPanel = new JPanel();
        JPanel jpDataBase = new JPanel(new BorderLayout());
        JPanel jpDataBaseMain = new JPanel(new BorderLayout());
        JPanel jpTable = new JPanel(new BorderLayout());

        JPanel jplo = new JPanel();
        JPanel jpro = new JPanel();
        JPanel jpld = new JPanel();
        JPanel jprd = new JPanel();

        table = new JTable();
        tableModel = new ATModel(gp);
        table.setModel(tableModel);
        table.setDefaultEditor(
            table.getColumnClass(1),
            new PropertyCellEditor());
        DefaultTableCellRenderer colorColumnRenderer =
            new DefaultTableCellRenderer();
        colorColumnRenderer.setBackground(Color.pink);
        table.setCellSelectionEnabled(false);
        TableColumn mColumn = table.getColumn("Label");
        mColumn.setMinWidth(200);
        mColumn.setCellRenderer(colorColumnRenderer);

        scrollPane = new JScrollPane(table);
        scrollPane.setPreferredSize(new Dimension(100, 240));
        scrollPane.setMaximumSize(new Dimension(100, 240));
        insertRow = new JButton("Insert Row");
        insertRow.addActionListener(this);

        deleteRow = new JButton("Delete Row");
        deleteRow.addActionListener(this);

        obServerUrl = new JTextField(20);
        obUserName = new JTextField(20);
        dbServerUrl = new JTextField(20);
        dbServerId = new JTextField(20);
        dbUserName = new JTextField(20);

        obsStatReq = new JCheckBox("Display OB Status (from database)");
        obsStatReq.addActionListener(DBPanel.this);
        databaseReq = new JCheckBox("Read observations from database");
        databaseReq.addActionListener(DBPanel.this);

        buttonsPanel.add(insertRow);
        buttonsPanel.add(deleteRow);

        jpTable.add(scrollPane, "North");
        jpTable.add(buttonsPanel, "South");

        // now start filling the panel left
        jplo.setLayout(new GridLayout(2, 1));
        jplo.setBorder(new EmptyBorder(3, 3, 3, 3));

        jpro.setLayout(new GridLayout(2, 1));
        jpro.setBorder(new EmptyBorder(3, 3, 3, 3));

        jplo.add(new JLabel("server URL:", SwingConstants.RIGHT));
        jplo.add(new JLabel("username:", SwingConstants.RIGHT));

        jpro.add(obServerUrl);
        jpro.add(obUserName);
        addToManaged(obServerUrl, obManagedVector);
        addToManaged(obUserName, obManagedVector);
        //addToManaged(table,obManagedVector);

        jpDataBase.add(jplo, BorderLayout.WEST);
        jpDataBase.add(jpro, BorderLayout.CENTER);

        jpDataBaseMain.setBorder(new EmptyBorder(10, 3, 20, 3));
        jpDataBaseMain.add(obsStatReq, BorderLayout.NORTH);
        jpDataBaseMain.add(jpDataBase, BorderLayout.SOUTH);

        setLayout(new BorderLayout());
        setBorder(new EmptyBorder(5, 5, 5, 5));

        add(jpDataBaseMain, "Center");
        add(jpTable, "South");

        buttonsPanel = new JPanel();
        jpDataBase = new JPanel(new BorderLayout());
        jpDataBaseMain = new JPanel(new BorderLayout());
        jpDataBaseMain.setBorder(new EmptyBorder(10, 3, 10, 3));

        // now start filling the panel left
        jpld.setLayout(new GridLayout(3, 1));
        jpld.setBorder(new EmptyBorder(3, 3, 3, 3));

        jprd.setLayout(new GridLayout(3, 1));
        jprd.setBorder(new EmptyBorder(3, 3, 3, 3));

        jpld.add(new JLabel("server URL:", SwingConstants.RIGHT));
        jpld.add(new JLabel("server ID:", SwingConstants.RIGHT));
        jpld.add(new JLabel("username:", SwingConstants.RIGHT));

        jprd.add(dbServerUrl);
        jprd.add(dbServerId);
        jprd.add(dbUserName);
        addToManaged(dbServerUrl, dbManagedVector);
        addToManaged(dbServerId, dbManagedVector);
        addToManaged(dbUserName, dbManagedVector);

        jpDataBase.add(jpld, BorderLayout.WEST);
        jpDataBase.add(jprd, BorderLayout.CENTER);

        jpDataBaseMain.add(databaseReq, BorderLayout.NORTH);
        jpDataBaseMain.add(jpDataBase, BorderLayout.SOUTH);

        add(jpDataBaseMain, "North");
        loadProperties(gp);

    }

    public void loadProperties(GasProp gp) {
        gpi = gp;
        String urlString;

        // MAPPING
        table.getDefaultEditor(table.getColumnClass(0)).cancelCellEditing();
        tableModel.loadProperties(gp);

        // OB STATUS DATABASE
        if (gp.getProperty("OB_STATUS_FROM_DB_REQUIRED") != null
            && gp.getProperty("OB_STATUS_FROM_DB_REQUIRED").equals("true")) {
            obsStatReq.setSelected(true);
            for (int i = 0; i < obManagedVector.size(); i++) {
                JComponent placeHolder = null;
                placeHolder = (JComponent) obManagedVector.elementAt(i);
                placeHolder.setEnabled(true);
            }
        }
        urlString = gp.getProperty("DB_SERVER_URL");
        if (urlString.startsWith("jdbc:sybase:Tds:")) {
            obServerUrl.setText(urlString.substring(16, urlString.length()));
        } else {
            obServerUrl.setText(urlString);
        }
        obUserName.setText(gp.getProperty("DB_USER_NAME"));

        // OBSERVATION DATABASE
        if (gp.getProperty("DBOBS_REQUIRED") != null
            && gp.getProperty("DBOBS_REQUIRED").equals("true")) {
            databaseReq.setSelected(true);
            for (int i = 0; i < dbManagedVector.size(); i++) {
                JComponent placeHolder = null;
                placeHolder = (JComponent) dbManagedVector.elementAt(i);
                placeHolder.setEnabled(true);
            }
        }
        urlString = gp.getProperty("DBOBS_SERVER_URL");
        if (urlString.startsWith("jdbc:sybase:Tds:")) {
            dbServerUrl.setText(urlString.substring(16, urlString.length()));
        } else {
            dbServerUrl.setText(urlString);
        }

        //dbServerUrl.setText(gp.getProperty( "DBOBS_SERVER_URL" ));
        dbServerId.setText(gp.getProperty("DBOBS_SERVER_ID"));
        dbUserName.setText(gp.getProperty("DBOBS_USERID"));
    }

    private void addToManaged(JComponent jc, Vector manVect) {
        jc.setEnabled(false);
        manVect.addElement(jc);
    }

    public void actionPerformed(ActionEvent e) {
        JComponent placeHolder = null;
        if (e.getSource() == deleteRow) {
            dbPanelDeleteSelectedRow();
        } else if (e.getSource() == insertRow) {
            dbPanelInsertBlankRow();
        }

        // here we have got to differentiate between save and load
        if (e.getSource() == obsStatReq) {
            if (obsStatReq.isSelected()) {
                gpi.put("OB_STATUS_FROM_DB_REQUIRED", "true");
                for (int i = 0; i < obManagedVector.size(); i++) {
                    placeHolder = (JComponent) obManagedVector.elementAt(i);
                    placeHolder.setEnabled(true);
                }
            } else {
                gpi.put("OB_STATUS_FROM_DB_REQUIRED", "false");
                for (int i = 0; i < obManagedVector.size(); i++) {
                    placeHolder = (JComponent) obManagedVector.elementAt(i);
                    placeHolder.setEnabled(false);
                }
            }
        }

        // here we have got to differentiate between save and load
        if (e.getSource() == databaseReq) {
            if (databaseReq.isSelected()) {
                gpi.put("DBOBS_REQUIRED", "true");
                for (int i = 0; i < dbManagedVector.size(); i++) {
                    placeHolder = (JComponent) dbManagedVector.elementAt(i);
                    placeHolder.setEnabled(true);
                }
            } else {
                gpi.put("DBOBS_REQUIRED", "false");
                for (int i = 0; i < dbManagedVector.size(); i++) {
                    placeHolder = (JComponent) dbManagedVector.elementAt(i);
                    placeHolder.setEnabled(false);
                }
            }
        }

    }

    /*
     * Save the properties from the GUI to the properties database.
     * 
     * @return String null on success, non-null on error - the returned
     * String is the error message.
     */
    public String saveProperties() {
        String rc = null;
        String urlString;
        table.getDefaultEditor(table.getColumnClass(0)).stopCellEditing();

        urlString = obServerUrl.getText();
        if (!urlString.startsWith("jdbc:sybase:Tds:")) {
            gpi.put("DB_SERVER_URL", "jdbc:sybase:Tds:" + urlString);
        } else {
            gpi.put("DB_SERVER_URL", urlString);
        }

        gpi.put("DB_USER_NAME", obUserName.getText());
        finishEditing();

        urlString = dbServerUrl.getText();
        if (!urlString.startsWith("jdbc:sybase:Tds:")) {
            gpi.put("DBOBS_SERVER_URL", "jdbc:sybase:Tds:" + urlString);
        } else {
            gpi.put("DBOBS_SERVER_URL", urlString);
        }

        gpi.put("DBOBS_SERVER_ID", dbServerId.getText());
        gpi.put("DBOBS_USERID", dbUserName.getText());
        if (databaseReq.isSelected()
            && (dbServerUrl.getText().length() == 0
                || dbUserName.getText().length() == 0
                || dbServerId.getText().length() == 0)) {

            rc =
                "One or more of the required observations database fields is blank.";
        }
        if (obsStatReq.isSelected()
            && (obServerUrl.getText().length() == 0
                || obUserName.getText().length() == 0)) {

            rc =
                "One or more of the required OB Status database fields is blank.";
        }
        return rc;
    }

    private void dbPanelDeleteSelectedRow() {
        int selRow = table.getSelectedRow();
        if (selRow < 0) {
            return;
        }
        tableModel.removeRow(selRow);
    }

    private void dbPanelInsertBlankRow() {
        int selRow = table.getSelectedRow();
        if (selRow < 0) {
            return;
        }
        tableModel.insertRow(selRow);
    }

    public void editingStopped(ChangeEvent e) {
    }

    public void editingCanceled(ChangeEvent e) {
    }

    public void finishEditing() {
        if (table.isEditing()) {
            TableCellEditor comp = table.getCellEditor();
            if (comp != null && comp instanceof DefaultCellEditor) {
                ((DefaultCellEditor) comp).stopCellEditing();
            }
            table.clearSelection();
        }
    }
    
    /**
     * @see org.eso.gasgano.PropertyEditorSubPanel#restartRequired()
     */
    public int restartRequired() {
        return NOACTION_ON_SAVE;
    }
}