package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.text.AttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

/**
 * class FontChooser, thought to be used within a JDialog:
 * most useful  method is Font getNewFont();
 * gets a Font as argument in the constructor
 * 
 * it merely shows a Component with two buttons 
 * and two comboBoxes allowing to specify
 * font family, size and style
 *
 * @author Michele Zamparelli
 */

public class FontChooser extends JComponent implements ActionListener {

    static private final String[] sizes =
        { "3", "4", "6", "8", "10", "12", "14", "16", "18" };
    private JComboBox fontName = null;
    private JCheckBox fontBold, fontItalic;
    private JComboBox fontSize = null;
    private JLabel previewLabel = null;
    private SimpleAttributeSet attributes = null;
    private Font newFont = null;

    // constructor
    public FontChooser(Font f) {
        setLayout(new BorderLayout());
        attributes = new SimpleAttributeSet();
        String[] fonts = null;
        JPanel fontPanel = new JPanel();

        // first we try to set the default font
        newFont = f;
        // here we should check that the initial font is in the list
        // otherwise we should modify it
        // System.out.println("default font: "+ f.getFamily());

        StyleConstants.setFontSize(attributes, f.getSize());
        StyleConstants.setFontFamily(attributes, f.getFamily());

        StyleConstants.setBold(attributes, f.isBold());
        StyleConstants.setItalic(attributes, f.isItalic());
        // 

        try {
            // fonts = Toolkit.getDefaultToolkit().getFontList();
            fonts =
                GraphicsEnvironment
                    .getLocalGraphicsEnvironment()
                    .getAvailableFontFamilyNames();
        } catch (Exception e) {
            // System.out.println("FontChooser: could not get font list from Toolkit");
            // System.out.println(e.toString());
            // something's missing here
        }

        fontName = new JComboBox(fonts);
        fontName.setSelectedItem(f.getFamily());
        fontName.addActionListener(this);

        fontSize = new JComboBox(sizes);
        fontSize.setSelectedItem(String.valueOf(f.getSize()));
        fontSize.addActionListener(this);

        fontBold = new JCheckBox("Bold");
        fontBold.addActionListener(this);
        if (f.isBold()) {
            fontBold.setSelected(true);
        }

        fontItalic = new JCheckBox("Italic");
        fontItalic.addActionListener(this);
        if (f.isItalic()) {
            fontItalic.setSelected(true);
        }

        fontPanel.add(fontName);
        fontPanel.add(new JLabel("  Size:  "));
        fontPanel.add(fontSize);
        fontPanel.add(fontBold);
        fontPanel.add(fontItalic);

        add(fontPanel, BorderLayout.NORTH);

        JPanel previewPanel = new JPanel(new BorderLayout());
        previewLabel = new JLabel("  Here's a sample of this font.");
        previewLabel.setFont(f); // set the inital font
        previewPanel.add(previewLabel, BorderLayout.CENTER);

        // give the preview label room to grow

        previewPanel.setMinimumSize(new Dimension(100, 100));
        previewPanel.setPreferredSize(new Dimension(100, 100));

        add(previewPanel, BorderLayout.SOUTH);
    }

    // now some methods

    // ok something in the font changed, so figure that out and make a 
    // new font for the preview label

    public void actionPerformed(ActionEvent ae) {
        // check the name of the font
        if (!StyleConstants
            .getFontFamily(attributes)
            .equals(fontName.getSelectedItem())) {
            StyleConstants.setFontFamily(
                attributes,
                (String) fontName.getSelectedItem());
        }

        // check the font size, no error checking yet
        if (StyleConstants.getFontSize(attributes)
            != Integer.parseInt((String) (fontSize.getSelectedItem()))) {
            StyleConstants.setFontSize(
                attributes,
                Integer.parseInt((String) (fontSize.getSelectedItem())));
        }
        // check to see if the font should be bold
        if (StyleConstants.isBold(attributes) != fontBold.isSelected()) {
            StyleConstants.setBold(attributes, fontBold.isSelected());
        }
        // check to see if the font should be Italic
        if (StyleConstants.isItalic(attributes) != fontItalic.isSelected()) {
            StyleConstants.setItalic(attributes, fontItalic.isSelected());
        }

        updatePreviewFont();

    }

    // get the appropriate color from our chooser and update previewLabel
    protected void updatePreviewFont() {
        String name = StyleConstants.getFontFamily(attributes);
        boolean bold = StyleConstants.isBold(attributes);
        boolean ital = StyleConstants.isItalic(attributes);
        int size = StyleConstants.getFontSize(attributes);

        Font f =
            new Font(
                name,
                (bold ? Font.BOLD : 0) + (ital ? Font.ITALIC : 0),
                size);
        previewLabel.setFont(f);
        previewLabel.repaint();
        newFont = f;
    }

    public Font getNewFont() {
        return newFont;
    }

    public AttributeSet getAttributes() {
        return attributes;
    }

    public static void main(String args[]) {
        JFrame frame = new JFrame("test mike font chooser");
        frame.setSize(100, 100);
        frame.pack();
        frame.setVisible(true);
        FontChooser fc =
            new FontChooser(
                new Font("SansSerif", Font.PLAIN + Font.ITALIC, 18));

        if (JOptionPane
            .showConfirmDialog(
                frame,
                fc,
                "Edit fonts",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.PLAIN_MESSAGE)
            == JOptionPane.OK_OPTION) {
            System.out.println(
                "selected Font is:" + fc.getNewFont().toString());
        }
        System.exit(0);
    }

}

// end of file FontChooser.java
