package org.eso.gasgano.gui;

import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

/**
 * @author Nick Kornweibel
 *
 * A table model for viewing and editing an underlying List of Leywords - the table
 * model, being a list, has only a single column.
 */
public class ListAdaptor extends AbstractTableModel {

    private List data;
    private String title;

    /**
     * Constructor
     * @param d the data to view and edit.
     * @param t the title of the column as appears in the table
     */
    public ListAdaptor(List d, String t) {
        this(d, t, null);
    }
    
    /**
     * Constructor
     * @param d the data to view and edit.
     * @param t the title of the column as appears in the table
     * @param val if the list is empty, add one row using the
     * given cell value (nicer for the user to edit).
     */   
    public ListAdaptor(List d, String t, Object val) {
        data = d;
        title = t;
        if (val != null && data.size() == 0) {
            insertRow(0, val);
        }
    }
    
    /**
     * Set the data to display.
     * @param d
     */
    public void setData(List d) {
        data = d;
        fireTableDataChanged();
    }

    public int getColumnCount() {
        return 1;
    }

    public int getRowCount() {
        return data.size();
    }

    public int getColumnWidth(int column) {
        if (column == 0) {
            return 240;
        }
        return 0;
    }

    public Object getValueAt(int row, int col) {
        return data.get(row).toString();
    }

    public boolean isCellEditable(int row, int col) {
        return true;
    }

    public String getColumnName(int column) {
        return title;
    }
    
    /**
     * Override this method to customize cell input validation.
     */
    public boolean validCellValue(Object aValue, int row, int column) {
        return true;
    }

    public void setValueAt(Object aValue, int row, int column) {
        if (column == 0) {
            if (validCellValue(aValue, row, column)) {
                data.set(row, (String) aValue);
            }
        }
    }

    public void removeRow(int rowNum) {
        try {
            data.remove(rowNum);
            // this is the only way I can get the JTable to update correctly
            // after the change - this call should not be necessay (fireTableDataChanged())
            // should be enough, but it does not seem to do the trick.
            fireTableStructureChanged();
        } catch (ArrayIndexOutOfBoundsException e) {
            e.printStackTrace();
        }
    }

    public void insertRow(int rowNum, Object val) {
        try {
            data.add(rowNum, val);
            // this is the only way I can get the JTable to update correctly
            // after the change - this call should not be necessay (fireTableDataChanged())
            // should be enough, but it does not seem to do the trick.
            fireTableStructureChanged();
        } catch (ArrayIndexOutOfBoundsException e) {
            e.printStackTrace();
        }
    }
    
    public void appendRow(Object val) {
        data.add(val);
        fireTableDataChanged();
    }

    public Class getColumnClass(int col) {
        return getValueAt(0, 0).getClass();
    }
}
