/**
 * @author N. Kornweibel
 * @version 1.0
 * Implementation of the AND operator for the Gasgano Keyword expression set.
 */

package org.eso.gasgano.keyword;

import java.util.Enumeration;

public class AndKeywordExpr extends DefaultBoolKeywordExpr {

    static final long serialVersionUID = -6460222304348137131L;
    
    public static final String opString = "AND";

    public AndKeywordExpr() { }

    public boolean evaluate(KeywordSet s) 
            throws ExpressionEvaluationException {
        boolean rc = true;
        if (getChildCount() < 2) {
            throw new ExpressionEvaluationException
                ("AND expression requires at least two arguments");
        } else {
            Enumeration e = children();
            while (e.hasMoreElements()) {
                BoolKeywordExpr expr = (BoolKeywordExpr)e.nextElement();
                rc = expr.evaluate(s);
                if (rc == false) {
                    break;
                }
            }
        }
        return rc;
    }

    public String getOpString() {
        return opString;
    }
    
    /**
     * @see org.eso.gasgano.keyword.DefaultBoolKeywordExpr#copy()
     */
    public DefaultBoolKeywordExpr copy() {
        DefaultBoolKeywordExpr rc = new AndKeywordExpr();
        Enumeration e = children();
        while (e.hasMoreElements()) {
            DefaultBoolKeywordExpr expr = (DefaultBoolKeywordExpr)e.nextElement();
            rc.add(expr.copy());
        }
        return rc;
    }
}
// end of file AndKeywordExpr.java
