package org.eso.gasgano.keyword;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

/**
 * @author nkornwei
 */
public class KeywordBlock implements KeywordSet {

    /**
     * List of all keywords in the block (used to maintain keyword ordering)
     */
    private List kArray;
    
    /**
     * Dictionary of all keywords in the block (used for quick lookup).
     */
    private Dictionary kDic; 

    /**
     * Create an empty keyword block
     * @see java.lang.Object#Object()
     */
    public KeywordBlock() {
        kArray = new Vector();
        kDic = new Hashtable();
    }
    
    /**
     * Create a keyword block from the provided kwordList and kwordDic.
     * The list and dictionary are not copied, they are simply referenced.
     * It is assumed that the kwordList and kwordDic have the same number of
     * elements and are built from the same keywords.
     * @param kwordList
     * @param kwordDic
     */
    public KeywordBlock(List kwordList,Dictionary kwordDic) {
        kArray = kwordList;
        kDic = kwordDic;
    }

    /**
     * @see org.eso.gasgano.KeywordSet#addKeyword(org.eso.gasgano.Keyword)
     */
    public void addKeyword(Keyword k) {
        if (k != null) {
            try {
                kDic.put(k.getName(), k);
            } catch (NullPointerException ex) {
                System.out.println("error:" + ex.toString());
            }
            kArray.add(k);
        }
    }

    /**
     * @see org.eso.gasgano.KeywordSet#getKeyword(java.lang.String)
     */
    public Keyword getKeyword(String name) {
        Keyword rc = null;
        rc = (Keyword)kDic.get(name);
        return rc;
    }

    /**
     * @see org.eso.gasgano.KeywordSet#removeKeyword(java.lang.String)
     * Remove the first occurrence of the keyword with the same FULL name
     * as the name specified. Duplicates are not searched for or removed.
     */
    public void removeKeyword(String name) {
        kDic.remove(name);
        for (int i = 0; i < kArray.size(); i++) {
            Keyword kw = (Keyword)kArray.get(i);
            if (kw != null && kw.getFullName().equals(name)) {
                kArray.remove(i);
                break;
            }
        }
    }
    
    
    /**
     * @see org.eso.gasgano.KeywordSet#replaceKeyword(java.lang.String)
     * Replace the first occurrence of the keyword with the same FULL name
     * as the name of the given keyword.
     * Duplicates are not searched for or replaced.
     */
    public void replaceKeyword(Keyword k) {
        kDic.put(k.getName(), k);
        for (int i = 0; i < kArray.size(); i++) {
            Keyword kw = (Keyword)kArray.get(i);
            if (kw != null && kw.getName().equals(k.getName())) {
                kArray.set(i, k);
                break;
            }
        }
    }

    /**
     * @return List the list of keywords in the block
     */
    public List getList() {
        return kArray;
    }
    
    /**
     * Get keyword at specified index.
     * @param i index of Keyword to get
     * @return Keyword
     */
    public Keyword getKeywordAt(int i) {
        return (Keyword)kArray.get(i);
    }
    
    /**
     * 
     * @return int number of keywords in the block
     */
    public int size() {
        return kArray.size();
    }
}
