package org.eso.gasgano.properties;

import org.eso.gasgano.keyword.Keyword;

/**
 * A table model for editing menu item properties.
 * @author nkornwei
 *
 */

public class MenuTableModel extends PropertyTableModel {
    // instance variables 
    private GasProp gpx = null;
    private String[] menuLabels = new String[4];
    private String[] menuCommands = new String[4];
    private Object[][] tableContent;
    private String retrieveCommand;

    public MenuTableModel(GasProp gp) {
        super();
        loadProperties(gp);
    }

    /**
     * Read the properties and apply them to the tableContent.
     * @param gp
     */
    public void loadProperties(GasProp gp) {
        gpx = gp;
        // 

        menuLabels[0] = (String) gpx.getProperty("MENU_CUSTOM1_LBL");
        menuLabels[1] = (String) gpx.getProperty("MENU_CUSTOM2_LBL");
        menuLabels[2] = (String) gpx.getProperty("MENU_CUSTOM3_LBL");
        menuLabels[3] = (String) gpx.getProperty("MENU_CUSTOM4_LBL");

        menuCommands[0] = (String) gpx.getProperty("MENU_CUSTOM1");
        menuCommands[1] = (String) gpx.getProperty("MENU_CUSTOM2");
        menuCommands[2] = (String) gpx.getProperty("MENU_CUSTOM3");
        menuCommands[3] = (String) gpx.getProperty("MENU_CUSTOM4");

        retrieveCommand = (String) gpx.getProperty("MENU_RETRIEVE");

        Object[][] trickArray =
            { { "Custom Menu 1", menuLabels[0], menuCommands[0] }, {
                "Custom Menu 2", menuLabels[1], menuCommands[1] }, {
                "Custom Menu 3", menuLabels[2], menuCommands[2] }, {
                "Custom Menu 4", menuLabels[3], menuCommands[3] }, {
                "Retrieve Files", "Retrieve Files", retrieveCommand }
        };
        tableContent = trickArray;
        fireTableDataChanged();
    }

    public int getColumnCount() {
        return 3;
    }

    public int getRowCount() {
        return 5;
    }

    public int getColumnWidth(int column) {
        if (column == 0 || column == 2) {
            return 20;
        }
        return 50;
    }

    public Object getValueAt(int row, int col) {
        Object rc = tableContent[row][col];
        return rc;
    }

    public boolean isCellEditable(int row, int col) {
        if ((col == 1 && row == 4) || (col == 0)) {
            return false;
        }
        return true;
    }

    public String getColumnName(int column) {
        String rc = null;
        if (column == 0) {
            rc = "";
        }
        if (column == 1) {
            rc = "label";
        }
        if (column == 2) {
            rc = "command";
        }
        return rc;
    }

    public void setValueAt(Object aValue, int row, int column) {
        tableContent[row][column] = (String) aValue;
    }

    public Class getColumnClass(int col) {
        Class cl = null;
        switch (col) {
            case 0 :
                cl = String.class;
                break;
            case 1 :
                cl = String.class;
                break;
            case 2 :
                cl = String.class;
                break;
        }
        return cl;
    }

    public void removeRow(int rowNum) {
        // do nothing - can not remove rows.
    }

    public void insertRow(int rowNum) {
        // can not insert rows.
    }

    /**
     * Get the table contents as a table of Objects.
     * @return Object[][]
     */
    public Object[][] getTableContent() {
        return tableContent;
    }
    
    /**
     * Set the table contents from the table of Objects given.
     * @param content
     */
    public void setTableContent(Object[][] content) {
        tableContent = content;
    }

    /**
     * Returns the retrieveCommand.
     * @return String
     */
    public String getRetrieveCommand() {
        return retrieveCommand;
    }

    /**
     * Sets the retrieveCommand.
     * @param retrieveCommand The retrieveCommand to set
     */
    public void setRetrieveCommand(String retrieveCommand) {
        this.retrieveCommand = retrieveCommand;
    }

}
