package org.eso.gasgano.tools;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

import org.eso.gasgano.properties.PropertyDB;

/**
 * @author nkornwei
 *
 * Tools for running shell commands.
 */
public class Exec {

    public static int runCommand(String command) {
        int rc = -1;
        String cmnds[] = new String[3];
        String command2 = cleverProcessCommand(command);

        cmnds[0] = "sh";
        cmnds[1] = "-c";
        cmnds[2] = command2;
        try {
            Runtime r = Runtime.getRuntime();
            Process p = r.exec(cmnds);
            try {
                p.waitFor();
                rc = p.exitValue();
            } catch (InterruptedException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
            } catch (IllegalThreadStateException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
            }
        } catch (IOException e) {
            System.out.println("Error executing command: " + e.toString());
        }
        return rc;
    }


    /**
     * this method is not ready yet, do not use
     *
     */
    public static int runCommandInBackground(String command) {
        int rc = -1;
        String cmnds[] = new String[3];
        String command2 = cleverProcessCommand(command);

        cmnds[0] = "sh";
        cmnds[1] = "-c";
        cmnds[2] = command2;
        try {
            Runtime r = Runtime.getRuntime();
            Process p = r.exec(cmnds);
            try {
                p.waitFor();
                rc = p.exitValue();
            } catch (InterruptedException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
            } catch (IllegalThreadStateException ie) {
                System.out.println(
                    "Interrupted while executing: " + ie.toString());
            }
        } catch (IOException e) {
            System.out.println("Error executing command: " + e.toString());
        }
        return rc;
    }

    /**
     * tries to shorten the command string by analyzing 
     * whether all the argument files mentioned have  a common<p>
     * directory. If they do, a 'cd' to that directory is made prior
     * to executing the original command<p>
     *
     * as of now, it doesn't have provisions for dealing with 
     * parenthesis or initial shell encapsulation<p>
     * @param: command string to be analyzed and shortened
     *
     **/

    public static String cleverProcessCommand(String longForm) {
        StringBuffer shortForm = new StringBuffer();
        boolean mvOrCp = false;
        boolean commonBase = true;
        boolean destDirOn = false;
        String commonDirectory = null;

        if (PropertyDB.getInstance().getProperty("SHORTEN_FILES_PATH").equals("false")) {
            return longForm;
        }

        int firstPlace = longForm.indexOf(' ');
        if (firstPlace == -1) {
            System.out.println("Could not identify command");
            return longForm;
        }
        String command = longForm.substring(0, firstPlace);
        if (command.equals("mv") || command.equals("cp")) {
            mvOrCp = true;
        }
        for (int z = firstPlace; z < longForm.length() - 1 && z != -1;) {
            String fileName = null;
            File tmpFile = null;

            int nextPlace = longForm.indexOf(' ', z + 1);
            if (nextPlace == -1) {
                fileName = longForm.substring(z + 1, longForm.length());
                if (mvOrCp) {
                    destDirOn = true;
                }
            } else {
                fileName = longForm.substring(z + 1, nextPlace);
            }
            tmpFile = new File(fileName);
            // it could be a command for the database
            if (!tmpFile.exists()) {
                //System.out.println("File did not exist:(" + tmpFile + ") " + longForm);
                return longForm;
            }

            if (commonDirectory == null) {
                commonDirectory = tmpFile.getParent();
            }
            if (!tmpFile.getParent().equals(commonDirectory) && !destDirOn) {
                commonBase = false;
            }
            if (destDirOn) {
                shortForm.append(tmpFile);
            } else {
                shortForm.append(tmpFile.getName() + " ");
            }
            z = nextPlace;
        }

        if (commonBase) {
            shortForm.insert(
                0,
                "cd " + commonDirectory + " ;  " + command + " ");
            //System.out.println("Common base found"+shortForm);
            return shortForm.toString();
        }
        //System.out.println("Long form "+longForm);
        return longForm;
    }
    
    /**
     * Run a command capturing the stdout and stderr and returning the 
     * return code of the program.
     * @param commandString
     * @param errString
     * @param outString
     * @return int
     */
    public static int shellCommand(
        String commandString,
        Vector errString,
        Vector outString) {
        Process shellProcess = null;
        int exitValue = 0;
        String singleLine = null;

        errString.removeAllElements();
        outString.removeAllElements();

        try {
            shellProcess = Runtime.getRuntime().exec(commandString);
        } catch (IOException ioe) {
            // do nothing for the moment
            System.err.println(ioe.getMessage());
            errString.addElement(new String("shellCommand: IOException"));
            return 1;
        }

        if (shellProcess != null) {
            // process is running
            try {
                exitValue = shellProcess.waitFor();
            } catch (InterruptedException ie) {
                // process was interrupted
                System.err.println(
                    "shellCommand: spawned subshell was interrupted");
                return 1;
            }
            // process terminated
            try {
                if (exitValue != 0) {
                    BufferedReader buffRed =
                        new BufferedReader(
                            new InputStreamReader(
                                shellProcess.getErrorStream()));
                    while ((singleLine = buffRed.readLine()) != null) {
                        errString.addElement(singleLine);
                    }
                    return 1;
                } else {
                    BufferedReader buffRed =
                        new BufferedReader(
                            new InputStreamReader(
                                shellProcess.getInputStream()));
                    while ((singleLine = buffRed.readLine()) != null) {
                        outString.addElement(singleLine);
                    }
                    return 0;
                }
            } catch (IOException ioe) {
                // much ado for nothing
                System.err.println("Exception occurred:");
                System.err.println(ioe);
                errString.addElement(new String("IOException occurred"));
                return 1;
            }
        } else { // no shell process  
            errString.addElement(
                new String("shellCommand: no RunTime process allocated"));
            return 1;
        }
    }

}
