package org.eso.ohs.core.dbb.client;

import javax.swing.JFrame;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.gui.widgets.JSpinnerInt;

/**
 * This is an implementation of the DbbWidget abstract class,
 * defining a Spinner widget displaying integer values
 * <p>
 * $Id: DbbSpinnerInt.java,v 1.2 2004/02/17 12:44:06 tcanavan Exp $
 * <br>
 * @author Carlo Boarotto, ESO 08-Dec-1999
 */
public class DbbSpinnerInt extends DbbWidget {

    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Create a DbbWidget with a JSpinner
     *
     * @param labeltext Label text to be displayed
     * @param chunk     DbbSqlChunk object
     */
    public DbbSpinnerInt(JFrame parentFrame_,
            String labeltext, DbbSqlChunk chunk,
            int value, int extent, int min, int max, boolean wrap) {
        super(parentFrame_, labeltext, chunk);
        widget_ = new JSpinnerInt(value, extent, min, max, wrap);
        label_.setLabelFor(widget_);
    }


    /**
     * Create a DbbWidget with a JSpinner
     *
     * @param labeltext Label text to be displayed
     * @param chunk     DbbSqlChunk object
     */
    public DbbSpinnerInt(String labeltext, DbbSqlChunk chunk,
            int value, int extent, int min, int max, boolean wrap) {
        this(null, labeltext, chunk, value, extent, min, max, wrap);
    }


    /**
     * Create a DbbWidget with a JSpinner
     *
     * @param labeltext Label text to be displayed
     * @param chunk     DbbSqlChunk object
     */
    public DbbSpinnerInt(DbbSqlChunk chunk,
            int value, int extent, int min, int max, boolean wrap) {
        this(null, chunk.getLabel(), chunk, value, extent, min, max, wrap);
    }


    /**
     * Create a DbbWidget with a JSpinner
     *
     * @param labeltext Label text to be displayed
     * @param chunk     DbbSqlChunk object
     */
    public DbbSpinnerInt(JFrame parentFrame_, DbbSqlChunk chunk,
            int value, int extent, int min, int max, boolean wrap) {
        this(parentFrame_, chunk.getLabel(), chunk, value, extent, min, max, wrap);
    }


    //----------------------------------------
    // Public methods
    //----------------------------------------


    /**
     * Get the widget value
     *
     * @return A String value: <code>true</code> if selected,
     *         <code>false</code> otherwise
     */
    public String get() {
        return String.valueOf(((JSpinnerInt)widget_).getValue());
    }


    /**
     * Set the widget value
     *
     * @param val Value
     */
    public void set(String val) {
        ((JSpinnerInt)widget_).setValue(val);
    }


    /**
     * Reset the widget to its default state
     */
    public void reset() {
        ((JSpinnerInt)widget_).clear();
        fireDbbWidgetAction();
    }


    /** @return A boolean value: <code>true</code> if <em>not</em> selected
     *                           <code>false</code> otherwise
     */
    public boolean isEmpty() {
        return ((JSpinnerInt)widget_).isEmpty();
    }


    /**
     * set background color of this widget
     *
     * @param bgCol new background color
     */
    public void setBackground(java.awt.Color bgCol) {
        ((JSpinnerInt)widget_).setBackground(bgCol);
    }
}

