package org.eso.ohs.core.dbb.examples;

// $Id: ExampleBrowser2.java,v 1.3 2004/04/08 12:09:51 tcanavan Exp $

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.rmi.DbbLocalEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/** 
 * This class provides a three tier example. The server it runs
 * directly database query sending back the result to the client using
 * RMI. In this example the client is browsing the eso_users table on
 * the opc63 database.  <br>
 *
 * Make sure your <code>CLASSPATH</code> variable is correctly set.
 * <BR>
 *
 * The server DbbServer is supposed to be running on some host, and
 * connected to the test database on pclinnt1.hq.eso.org. In order to
 * run the example you must pass the URL of the middle-tier server on
 * the command line: 
<PRE> 
java org.eso.ohs.dbb.examples.ExampleBrowser2 rmi://<em>host</em>/<em>server</em>
</PRE>
 * where <em>host</em> is the name of the host where the server is
 * running and <em>server</em> is the server name. So, for instance, a
 * complete server URL would be: 
 * <BR>
 * <code>//usg1.hq.eso.org/DBB</code>:
 *
 * @see org.eso.ohs.core.dbb.server.DbbServer
 * @author Carlo Boarotto, ESO 10-Sep-2000
 * */
class ExampleBrowser2 extends JFrame {
private static final String rcsid = "$Id: ExampleBrowser2.java,v 1.3 2004/04/08 12:09:51 tcanavan Exp $";
private static Logger stdlog_ = 
    Logger.getLogger( ExampleBrowser2.class );

    /** RMI server url used to connect to the remote server */ 
    public static String RMI_SERVER_URL = "";
    
    /**
     * Associated sql engine reference (server). It must be an
     * interface and it has to be set creating a DbbLocalEngine that
     * connects (via RMI) the remote server. 
     */
    DbbSqlEngine sqle;

    /** Aggregated view (client) */
    DbbView view;

    /** First SQL table we are going to browse */
    DbbSqlTable table;

    /** Second SQL table we are going to browse */
    DbbSqlTable table2;
    
    /** Label for the "name" widgets */
    String labName  = "Name";

    /** Label for the "ID" widgets */
    String labID    = "UserID";

    /** Label for the "email" widgets */
    String labEmail = "E-mail";
    
    //----------------------------------------
    // Constructor
    //----------------------------------------
    
    /**
     * Create an object setting all its internal variable. The
     * constructor, invoking the private method buildSqlEngine,
     * creates the associated sql engine (DbbLocalEngine implements
     * the DbbSqlEngine interface) connecting to the RMI server.
     */
    public ExampleBrowser2() {
	// We browse the opc63..eso_users table
	table = new DbbSqlTable( "opc70", "", "eso_users" );
	table2 = new DbbSqlTable( "opc70", "", "user_affiliations" );
	
	buildSqlEngine();		
	view = new DbbView(sqle);
	
	buildColumns();
	buildRows();
	buildSort();
	
	getContentPane().add(view);
    } 

    
    //----------------------------------------
    // Private Methods
    //----------------------------------------
    
    /**
     * Build the SQL engine setting up a RMI connection to the server
     * and add the dafault sql chunks and join conditions.
     */
    private void buildSqlEngine () {

	try {
            
	    stdlog_.debug("DEBUG: Connecting the server via RMI: < " + 
			       RMI_SERVER_URL + 
			       " >");

	    /* NOTE: it must be a local engine created passing the RMI
             * server url because it has been delegated to set up the
             * connection to the RMI server 
	     */
	    sqle = new DbbLocalEngine(RMI_SERVER_URL);

	    /* Default column: user ID */
	    DbbSqlChunk def = 
		new DbbSqlChunk(table, "id", labID);

	    /* Add the default chunks */
 	    sqle.addDefaultColumnsChunk(def);

	    /* Add the default join conditions */
	    sqle.addJoinCondition(table, "id", table2, "user_id");

	    stdlog_.debug("DEBUG: Server connected: < " + 
			       RMI_SERVER_URL + 
			       " >");
      
	} catch( Exception e ) { 
	    stdlog_.warn("",e); 
	}
    }


    /**
     * Build the column selection panel. 
     *
     * The user ID column is always selected (see the call to
     * DbbSqlEngine.addDefaultColumnsChunk() in buildSqlEngine());
     * user can also select from the surname_dl and email columns.  
     */
    private void buildColumns() {

	DbbSqlChunk c;
	DbbCheckBox w;
	DbbSqlOperator op;

	// Panel has one rows of widgets
	DbbPanel panel = new DbbPanel("Selected Columns", 1);

	// Add the widgets
	try {

	    op = new DbbSqlOperator("=");

	    c = new DbbSqlChunk(table, "surname_dl", labName, op);
	    w = new DbbCheckBox(labName, c);
	    panel.addWidget(w);

	    c = new DbbSqlChunk(table, "email", labEmail, op);
	    w = new DbbCheckBox(labEmail, c);
	    panel.addWidget(w);
	}

	catch(Exception e) { 
	    stdlog_.warn("",e); 
	}

	/* Add this panel to the view as the column selection panel */
	view.setColumnsPanel(panel, "West");
    }


    /**
     * Build the rows selection panel.
     *
     * We allow the user to search by (exact) name and by country code
     */
    private void buildRows() {

	DbbSqlChunk c;
	DbbTextField w;
	DbbSqlOperator op;

	// Panel has one rows of widgets
	DbbPanel panel = new DbbPanel("Selection Criteria", 1);

	// Add the widget
	try {

	    op = new DbbSqlOperator("LIKE");
	    DbbSqlOperator op2 = new DbbSqlOperator("in");

	    c = new DbbSqlChunk(table, "surname_dl", labName, op);
	    w = new DbbTextField(labName , c);
	    panel.addWidget(w);


	    c = new DbbSqlChunk(table2, "country_code", "Country", op2);

	    String[] affiliationValues = {"B", "CH", "D", "DK",
					  "F", "I", "P", "S",
					  "ESO", "ESA", "RCH",
					  "UK", "USA", "OTHER"};

	    DbbSelectionList country = new DbbSelectionList("Country",
							    c,
							    affiliationValues);
	    panel.addWidget(country);
	
	} 
	catch( Exception e ) { 
	    stdlog_.warn("",e); 
	}

	/* Add this panel to the view as the rows selection panel; that
	 * is, the panel to define the where clause 
	 */
	view.setWherePanel(panel, "North");
    }



    /**
     * Build the row sorting selection panel.
     *
     * We allow the user to sort by ID and name
     */
    private void buildSort() {

	DbbSqlChunk c;
	DbbWidget w;

	// Panel has one rows of widgets
	DbbPanel panel = new DbbPanel("Sorting Criteria", 1);

	try {

	    c = new DbbSqlChunk(table, "id", labID);
	    w = new DbbArrowButton(labID, c);
	    panel.addWidget(w);

	    c = new DbbSqlChunk(table, "surname_dl", labName);
	    w = new DbbArrowButton(labName, c);
	    panel.addWidget(w);
	} 

	catch(Exception e) { 
	    stdlog_.warn("",e); 
	}

	/* Add this panel to the view as the rows sorting panel; that
	 * is, the panel to define the where clause 
	 */
	view.setSortPanel(panel, "West");
    }

    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /** Main method, to run directly the example. */
    public static void main(String[] args) {

	int len = args.length;
	String msg = 
	    "Usage: ExampleBrowser2 rmi_server_url";

	msg = msg +
	    "\n" + " - Example: ExampleBrowser2 //usg3.hq.eso.org/DBB";

	if(len < 1) {
	    stdlog_.debug(msg);
	    System.exit(1);
	}

	RMI_SERVER_URL = args[0];

	ExampleBrowser2 dbb = new ExampleBrowser2();
	dbb.setTitle("Example 2 - Remote Connection");

	WindowListener l = new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);
		}
	    };
      
	dbb.addWindowListener(l);
	dbb.pack();
	dbb.setVisible(true);
    }
}
