package org.eso.ohs.core.utilities;

import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;

/**
 * Utility methods for observing periods.
 * <p>
 * $Id: Period.java,v 1.17 2005/05/10 13:12:36 ddorigo Exp $<br>
 * 
 * @author Bob Kemp
 */

public class Period {

    private static Logger stdlog_ = Logger.getLogger(Period.class);

    /** ] Add start and additional period */
    public static final int STARTPERIOD = 60;

    public static final int ADDITIONAL = 10;

    public static final int ESO_ORIG = 1968;

    public static final int NUM_PERIODS_YEAR = 2;

    public static final int MIN_PERIOD = 0;

    public static final int MAX_PERIOD = 99;

    //----------------------------------------
    // Constructor
    //----------------------------------------

    public Period() {
    }

    /**
     * It calculates the number of hours in one night.
     * The odd periods are in winter time, the even ones in summer.
     * @param currPeriod period for the conversion
     * @return int returns 8 or 10 hours depending from the season.
     */
    public  int hoursInNight(int currPeriod) {
        int modPeriod = currPeriod % 2;
        int hoursConversion = 0;
        // id even period night per hours is 8
        if (modPeriod == 0) {
            hoursConversion = 8;
        } else {
            hoursConversion = 10;
        }
        return hoursConversion;
    }

    /**
     * Returns the current period, determined from today's date
     * 
     * @return current observing period, eg 64
     */
    public static int getPeriod() {
        Period currPeriod = new Period();
        return currPeriod.periodVal(new Date());
    }
    
    
    public static int getCurrentPeriod() {
        Period currPeriod = new Period();
        return currPeriod.periodVal(new Date());
    }
    

    /**
     * Return the observing period, determined from the date
     * 
     * @param date
     *            current date used for conversion
     * @return current observing period, eg 64
     */
    public int periodVal(Date date) {

        Calendar now = Calendar.getInstance();
        now.setTime(date);
        int currentPeriod = (now.get(Calendar.YEAR) - ESO_ORIG)
                * NUM_PERIODS_YEAR;

        // adjust for changes within the year (january is 0)
        if (now.get(Calendar.MONTH) >= 9) { // changes in October
            currentPeriod += 2;
        } else if (now.get(Calendar.MONTH) >= 3) { // changes in April
            currentPeriod += 1;
        }

        // this 'if' can probably be safely deleted
        if (currentPeriod > MAX_PERIOD || currentPeriod < MIN_PERIOD) {
            stdlog_.warn(now.get(Calendar.YEAR) + " wrong param date value: "
                    + "ESO periods start in year: " + ESO_ORIG
                    + " and maximum number of periods is " + MAX_PERIOD
                    + " . The current period is set to " + MIN_PERIOD);
            currentPeriod = MIN_PERIOD;
        }

        return currentPeriod;
    }
    /**
     * Return the start date of the period
     *
     * @param period 
     * @return Date
     */
    public static Date periodStartDateVal(int period) {
        Calendar cal = Calendar.getInstance();
        int day = 1;
        cal.clear();
        int year = period / NUM_PERIODS_YEAR + ESO_ORIG;

        //odd
        if (period % 2 > 0) {
            year = (int) Math.ceil(year);
            cal.set(Calendar.MONTH, Calendar.APRIL);
        }
        //even
        if (period % 2 == 0) {
            year = (int) Math.ceil(year) - 1;
            cal.set(Calendar.MONTH, Calendar.OCTOBER);
        }
        cal.set(Calendar.YEAR, year);
        cal.set(Calendar.DAY_OF_MONTH, day);
        return cal.getTime();
    }//startDateVal

    /**
     * Return the end date of the period
     *
     * @param period 
     * @return Date
     */
    public static Date periodEndDateVal(int period) {
        Calendar cal = Calendar.getInstance();
        cal.clear();
        int oddDay = 30;
        int evenDay = 31;

        int year = period / NUM_PERIODS_YEAR + ESO_ORIG;

        //odd
        if (period % 2 > 0) {
            year = (int) Math.ceil(year);
            cal.set(Calendar.MONTH, Calendar.SEPTEMBER);
            cal.set(Calendar.YEAR, year);
            cal.set(Calendar.DAY_OF_MONTH, oddDay);
        }
        //even
        if (period % 2 == 0) {
            year = (int) Math.ceil(year);
            cal.set(Calendar.MONTH, Calendar.MARCH);
            cal.set(Calendar.YEAR, year);
            cal.set(Calendar.DAY_OF_MONTH, evenDay);
        }
        return cal.getTime();
    }
}