package org.eso.ohs.instruments;

/**
 * This is the metadata class that describes the constraints 
 * This class stores all the validation neccessary in the application. 
 * @author tcanavan
 *
 */
public class InstrumentConstraintMetaData {
    private String pattern = "N\\A";

    private double minExclusive = Double.NaN;

    private double maxExclusive = Double.NaN;

    private double minInclusive = Double.NaN;

    private double maxInclusive = Double.NaN;
    
    private int maxLength = 0;


    private String type;

    private String baseType;

    private String[] allowedValues;

    private String defaultValue;

    /**
     * The list of allowsed values for a given attribue.
     * @return List of allowed values
     */
    public String[] getAllowedValues() {
        if (allowedValues == null)
            return new String[0];
        return allowedValues;
    }

    /**
     * The Max inclusive values
     * @return Max Inclusive Value 
     */
    public double getMaxInclusive() {
        return maxInclusive;
    }

    
    /**
     * Get Min Inclusive Value
     * @return MinInclusiveValue
     */
    public double getMinInclusive() {
        return minInclusive;
    }

    /**
     * Get the minExclusive value.
     * @return Get the MinExclusive value.
     */
    public double getMinExclusive() {
        return minExclusive;
    }
    
    /**
     * Get the MaxExclusive value.
     * @return Get the MaxExclusive value.
     */


    public double getMaxExclusive() {
        return maxExclusive;
    }

    /**
     * Return the regular expression string for validation.
     * @return Regular expression for validation.
     */
    public String getPattern() {
        return pattern;
    }

    /**
     * Return the type double,int,string etc..
     * @return Type of variable. 
     */
    public String getType() {
        return type;
    }

    /**
     * The set of allowed values.
     * @param strings
     */
    public void setAllowedValues(String[] strings) {
        allowedValues = strings;
    }

    /**
     * Set maxExclusive value
     * @param maxExclusive
     */
    public void setMaxExclusive(double maxExclusive) {
        this.maxExclusive = maxExclusive;
    }

    /**
     * Set minExclusive value
     * @param minExclusive Min Exclusive Value
     */
    public void setMinExclusive(double minExclusive) {
        this.minExclusive = minExclusive;
    }

    /**
     * Set minInclusive value
     * @param minInclusive
     */
    public void setMinInclusive(double minInclusive) {
        this.minInclusive = minInclusive;
    }

    /**
     * Set MaxInclusive
     * @param Set Max Inclusive value.
     */
    public void setMaxInclusive(double maxInclusive) {
        this.maxInclusive = maxInclusive;
    }
    
    /**
     * Verify a value for a specific type. 
     * @param value
     * @param type
     * @return boolean true or false.
     */

    public boolean verify(String value, String type) {
        if (type.equals("string"))
            return inAllowedValues(value);
        else {
            return inRange(new Double(value).doubleValue());
        }
    }

    public boolean inAllowedValues(String value) {
        
        for (int i = 0; i < allowedValues.length; i++) {
            if (allowedValues[i].equals(value)) {
                return true;
            }
        }
        return false;
    }

    public boolean inRange(double value) {
        if (value < minInclusive || value > maxInclusive)
            return false;
        return true;
    }

    /**
     * @param string
     */
    public void setPattern(String string) {
        pattern = string;
    }

    /**
     * @param string
     */
    public void setType(String string) {
        type = string;
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("Min Exlusive " + getMinExclusive());
        buf.append(" Max Exlusive " + getMaxExclusive());
        buf.append(" Min Inclusive " + getMinInclusive());
        buf.append(" Max Inclusive " + getMaxInclusive());
        buf.append("Pattern " + getPattern());
        for(int i=0;i<allowedValues.length;i++) {
            buf.append(allowedValues[i]);
        }        
        return buf.toString();
    }

    /**
     * Return the base type int,double etc...
     * @return The type describing the constraint.
     */
    public String getBaseType() {
        return baseType;
    }

    /**
     * Set the base type.
     * @param string
     */
    public void setBaseType(String string) {
        baseType = string;
    }

    /**
     * Returns the defaultValue.
     * @return String
     */
    public String getDefaultValue() {
        return defaultValue;
    }

    /**
     * Sets the defaultValue.
     * @param defaultValue The defaultValue to set
     */
    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    /**
     * Get the maximum length for this type
     * @return Maximum length
     */
    public int getMaxLength() {
        return maxLength;
    }

    /**
     * @param i
     */
    public void setMaxLength(int maxLength) {
        this.maxLength = maxLength;
    }

}
