package org.eso.ohs.instruments;

import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Instancaites a paramater class that represents a keyword. 
 * The string that contains the keyword value is checked for its validity.
 * <p>
 * $Id: KeywordParameter.java,v 1.4 2001/07/25 12:15:11 tcanavan Exp $
 * @author Tim Canavan
 */

public class KeywordParameter extends Parameter {

    static final long serialVersionUID = -8132009154805539178L;

    /**
     * List of keywords
     */

    private String[] keywordList_;

    /**
     * Constructor for keyword parameter
     */

    public KeywordParameter(ParamMetaData pMeta) {
        super(pMeta);
        setKeywordList();
    }

    public char getDBTypeCode() {
        return 'K';
    }


    /**
     * Tooltip text for KeywordParameter
     * @return tooltip text
     */
    public String getToolTipText() {
        boolean edit=false;
        if(keywordList_ != null) {
	   for(int i=0;i<keywordList_.length;i++) {
               if(keywordList_[i].equals("Special"))
                   edit=true;
           }
        }
        if(!edit)
            return "Select item from combobox";
        else
            return "Select item from combobox or add your own item and select";
    }    


    /**
     * Set keywordlist based on range and 
     * ant extra values to be entered.     
     */
  
    public void setKeywordList() {
        Vector keywordList = new Vector();
        String range = getRange();
        if(range == null) {
            keywordList_ = new String[0];
            return;
        }
        StringTokenizer token = new StringTokenizer(getRange());
        keywordList_ = new String[token.countTokens()];
        int i=0;
        while(token.hasMoreElements()) {
            String item = token.nextToken();
            keywordList.addElement(item);
            i++;
        }
        String value = getValue();
        if(value != null && value.length() > 0 && !value.equals("NODEFAULT")) {
            StringTokenizer valToken = new StringTokenizer(value);
            while(valToken.hasMoreElements()) {
	        String valueItem = valToken.nextToken();
                if(!keywordList.contains(valueItem)) {
                    keywordList.addElement(valueItem);
                }
            }
        }
        keywordList_ = new String[keywordList.size()];
        keywordList.copyInto(keywordList_);
    }

    /**
     * Return the list of valid keywords that can be selected
     * @return a list of valid keywords
     */

    public String [] getKeywordList () {
        return keywordList_;
    }

    /**
     * Add a keyword to the keyword lisr
     * @param keyword keyword to be added to keywordlist
     */

    public void addKeyword(String keyword) {
        String [] keywordList = new String[keywordList_.length+1];
        System.arraycopy(keywordList_,0,keywordList,0,keywordList_.length);
        keywordList[keywordList.length-1] = keyword;
        keywordList_ = keywordList;
    }

    /**
     * This method verifies a string is a keyword value
     * @param value String representing keyword values
     * @return OK,Range of allowed values.
     */

    public boolean verifyValue(String value) {
        String range = getRange();
        String defaultValue = getDefaultValue();
        if(value == null || value.length() <=0) {
	    return false;
        }
        if(value.startsWith("FILT") || value.startsWith("OPTI"))
             return true;
        if(range == null || range.length() == 0) {
            return false;
        }
        if(value.compareTo("NODEFAULT")==0)
            return false;
        if(keywordList_ != null) {
            for(int i=0;i<keywordList_.length;i++) {
                if(value.equals(keywordList_[i]))
                     return true;
            }
        }
        return false;
    }   

}











