package org.eso.ohs.instruments;

import java.io.Serializable;

/**
 * Class ReadmeAttributes This is the list of readme attributes entered by the
 * user.
 * 
 * TODO Javadoc
 * 
 * @version $Revision: 1.15 $
 * @author tcanavan, Nov 9, 2004
 */

// $Id: ReadmeAttribute.java,v 1.15 2005/01/12 09:33:29 tcanavan Exp $
public abstract class ReadmeAttribute implements Serializable {
    
    
    public static final String VERIFY_OK = "OK";
    
    public static final String VERIFY_INVALID = "INVALID";
    
    public static final String VERIFY_TRUNCATED = "DATA TRUNCATED";

    
    public static final String VERIFY_EMPTY = "Not Defined";
    
    private static final long serialVersionUID = -4378250031958131200L;

    private String name;
    
    
    private String type;


    /**
     * This stores the value of a given readme attribute.
     */

    private String value_;

    /**
     * Set the readmeConfiguration for a given
     */

    private transient InstrumentConstraintConfig readmeConfig_ = null;

    /**
     * Construct a readme attribure that contains the metadata for the
     * associated for a given
     * 
     * @param readmeConfig
     */

    public ReadmeAttribute(InstrumentConstraintConfig readmeConfig) {
        readmeConfig_ = readmeConfig;
        name = readmeConfig.getName();
        setValue(readmeConfig.getMetaData().getDefaultValue());
        setType(readmeConfig.getSuperType());
    }

    /**
     * @return Returns the value.
     */
    public String getValue() {
        if(value_ == null)
            return "";
        return value_.trim();
    }
    
    public String getDisplayValue() {
        return getValue();
    }
    
    public boolean hasValue() {
        String value = getValue();
        if(value != null && value.length()>0) {
            return true; 
        }
        return false;
    }

    /**
     * @param value
     *            The value to set.
     */
    public void setValue(String value) {
        this.value_ = value;
    }

    /**
     * @return Returns the readmeConfig_.
     */
    public InstrumentConstraintConfig getReadmeConfig() {
        return readmeConfig_;
    }
    
    /**
     * Return an XML string attribute for a string. 
     * @return returns the xml string for a given run. 
     */
    
    public String toReadmeAttributeString() {
        StringBuffer buf = new StringBuffer();
        buf.append("    <");
        buf.append(readmeConfig_.getName());
        buf.append(">");
        buf.append("\n");
        buf.append("        <Label>");
        buf.append(readmeConfig_.getLabel());
        buf.append("</Label>");
        buf.append("\n");
        buf.append("        <Data>");        
        buf.append(getValue());
        buf.append("</Data>");
        buf.append("\n");        
        buf.append("    </");
        buf.append(readmeConfig_.getName());
        buf.append(">");
        return buf.toString();        
    }


    /**
     * @param readmeConfig_
     * The readmeConfig_ to set.
     */
    public void setReadmeConfig(InstrumentConstraintConfig readmeConfig) {
        readmeConfig_ = readmeConfig;
    }

    /* (non-Javadoc)
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
        // TODO Auto-generated method stub
        
    }
    /**
     * The name of the attribute.
     * @return Name of attribute.
     */
    public String getName() {
        return name;
    }
    
    
    public abstract Object getRawValue();
    

    /**
     * @param string
     */
    public void setName(String string) {
        name = string;
    }
    
    public abstract boolean isCompulsory();
    
    public abstract String verify();
    
    public abstract String getToolTipText();
    
    //public static abstract String getDBType(); 


    

    /**
     * @return Returns the type.
     */
    public String getType() {
        return type;
    }
    /**
     * @param type The type to set.
     */
    public void setType(String type) {
        this.type = type;
    }
    
    public boolean equalsMeta(Object obj) {
        if(obj instanceof ReadmeAttribute) {
            ReadmeAttribute attrib=(ReadmeAttribute)obj;
            if(attrib.getType().equals(getType())
               && attrib.getName().equals(getName())) {
                return true;
            }
        }
        return false;
    }
}