package org.eso.ohs.persistence;

import java.io.Serializable;

/**
 * This class is used to define different media, eg local storage,
 * over the wire to ESO, directly to the database (used only at ESO?),
 * or to the local filesystem via the ImpEx format.
 * 
 * The reason for using a separate class, rather than
 * "static final int" variables, is that with this technique the
 * values are type-checked by the compiler.  Thus they are guaranteed
 * to be one of this set of values, else the code would not compile.
 *
 * <p>
 * $Id: Media.java,v 1.9 2003/07/03 08:49:49 ddorigo Exp $
 * <br>
 * @author Bob Kemp
 */

public class Media implements Serializable {

    /** Version id for object serialization */
    static final long serialVersionUID = 3692522654642069659L;

    /** Save the BusinessObject locally */
    public static final Media LOCAL       = new Media("local");

    /** Save the BusinessObject directly in the database */
    public static final Media PHASE2_DB   = new Media("ESO OBREP database");

    /** Backwards compatibility constant referring to OBREP database */
    public static final Media DBASE	  = PHASE2_DB;

    /** Save the BusinessObject directly in the database */
    public static final Media PHASE1_DB   = new Media("ESO OPC database");

    /** The name of the storage medium */
    private String name_;

    /**
      * Private constructor, so that there are a fixed number of
      * different media and all are defined by this class.
      * @param name name of media "local","database"
      *
      */
    private Media(String name) {
	name_ = name;
    }

    /**
     * Returns the descriptive text for this Media type.
     * 
     * @return description name.
     */
    public String getName() {
	return name_;
    }

    /**
     * Returns a string representation of this object and its fields.
     * 
     * @return a string representation.
     */
    public String toString() {
	return "Media[name=" + name_ + "]";
    }

    public int hashCode() {
	return name_.hashCode();
    }

    /**
     * Returns true if the given object is equal to this one.
     * 
     * @param obj	an object to compare with this one.
     * @return true if the given object is equal to this one.
     */
    public boolean equals(Object obj) {
	if (obj instanceof Media) {
	    Media device = (Media)obj;
	    return name_.equals(device.name_);
	}
	return false;
    }

}

