package org.eso.ohs.persistence.dbase.phase2;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
/**
 * Dbase operations for p2pp when reading,writing,deletion,updating
 * Remember that the actual finding chart data is removed from 
 * finding_charts only when the run is marked complete. 
 * 
 * 
 * @author tcanavan
 *
 */
public class FindingChartsDBIO {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(FindingChartsDBIO.class);

    private static String obrepDbName =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    private static String opcDB_ =
        Config.getCfg().getDbName(ObservingRun.getSuffix());

    /**
     * Read finding charts from the database
     * @param conn
     * @param boId
     * @return FindingChart[]
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static FindingChart[] read(
        Connection conn,
        long boId,
        boolean includingImage)
        throws ObjectIOException, SQLException, IOException, ObjectNotFoundException {

        //String sqlText = "SET TEXTSIZE 1000000";

        Statement stmt = conn.createStatement();
        int textLenMax = AppConfig.getAppConfig().getMaxFindingChartSize();
        stmt.execute("set TEXTSIZE " + textLenMax);
        String sqlText =
            "SELECT "
                + obrepDbName
                + "..finds.fc_id from "
                + obrepDbName
                + "..finds where "
                + obrepDbName
                + "..finds.ob_id = ?";
        stdlog_.debug(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, boId);
        ResultSet rs = pstmt.executeQuery();
        Vector vec = new Vector();
        while (rs.next()) {
            Integer fcId = new Integer(rs.getInt(1));
            if (!vec.contains(fcId))
                vec.addElement(fcId);
        }
        if (!includingImage) {
            sqlText =
                "SELECT "
                    + obrepDbName
                    + "..finding_charts.fc_name,"
                    + obrepDbName
                    + "..finding_charts.fc_notes,"
                    + obrepDbName
                    + "..finding_charts.fc_checksum from "
                    + obrepDbName
                    + "..finding_charts where "
                    + obrepDbName
                    + "..finding_charts.fc_id = ?";
        } else {
            sqlText =
                "SELECT * from "
                    + obrepDbName
                    + "..finding_charts where "
                    + obrepDbName
                    + "..finding_charts.fc_id  = ?";
        }
        stdlog_.debug(sqlText);
        pstmt = conn.prepareStatement(sqlText);
        //FindingCharts fcCharts = new FindingCharts(vec.size());
        Vector newVec = new Vector();
        for (int i = 0; i < vec.size(); i++) {
            int fcId = ((Integer) vec.elementAt(i)).intValue();
            pstmt.setInt(1, fcId);
            rs = pstmt.executeQuery();
            while (rs.next()) {
                FindingChart fcChart = new FindingChart();
                fcChart.setFcName(rs.getString("fc_name").trim());
                String notes = rs.getString("fc_notes");
                if (notes != null) {
                    fcChart.setFcNotes(notes.trim());
                } else {
                    fcChart.setFcNotes("");
                }
                fcChart.setCheckSum(rs.getLong("fc_checksum"));
                if (includingImage) {
                    InputStream is = rs.getBinaryStream("fc_data");
                    fcChart.setFcData(
                        FileIOUtils.convertInputStreamToBinary(is));
                }
                newVec.addElement(fcChart);
            }
            rs.close();
        }
        FindingChart[] fcs = new FindingChart[newVec.size()];
        newVec.copyInto(fcs);
        return fcs;
    }

    /**
         * Write finding charts to the database.
         * This involves wrtiting to 2 database tables finds
         * & finding_charts
         * 
         * Here are the rules for finding charts in the database, 
         * 
         * @param conn
         * @param boId
         * @param runId
         * @param fcList
         * @throws SQLException
         * @throws ObjectIOException
         */
    public static void write(
        Connection conn,
        long boId,
        long runId,
        FindingChart[] fcList)
        throws SQLException, ObjectIOException {
        String observatory = getObservatory(conn, runId);

        for (int i = 0; i < fcList.length; i++) {
            long checksum = fcList[i].getCheckSum();
            boolean fileContentsExists = false;
            String sqlText =
                "SELECT fc_id,fc_name,fc_checksum,run_id FROM finding_charts where fc_checksum = "
                    + checksum
                    + " AND finding_charts.run_id = " + runId;
            stdlog_.debug(sqlText);
            Statement selStmt = conn.createStatement();
            ResultSet rs = selStmt.executeQuery(sqlText);
            long fcId = -1;
            long fcRunId = -1;
            String fcName = null;
            long fcChecksum = -1;
            while (rs.next()) {
                fileContentsExists = true;
                fcId = rs.getInt("fc_id");
                fcRunId = rs.getInt("run_id");
                fcName = rs.getString("fc_name").trim();
                fcChecksum = rs.getLong("fc_checksum");
            }
            rs.close();
            selStmt.close();
            if (fileContentsExists) {
                // If checksums are the same but the names are diffenet
                if (!fcList[i].getFcName().trim().equals(fcName.trim())
                    && fcList[i].getCheckSum() == fcChecksum) {
                    Vector vec = getOBIds(conn, fcId);
                    if (vec.size() > 0) {
                        StringBuffer buf = sameContentsMessage(i, fcName, vec);
                        throw new SQLException(buf.toString());
                    } else {
                        if (findingChartNameExists(conn,
                            fcList[i].getFcName(),runId)) {
                            StringBuffer buf =
                                sameNameMessage(fcList[i].getFcName());
                            throw new SQLException(buf.toString());
                        }
                        updateFindingChartName(
                            conn,
                            fcId,
                            fcList[i].getFcName());
                    }
                }
                updateFinds(conn, boId, fcId, observatory);
                continue;
            }
            sqlText =
                "SELECT fc_id,fc_name,fc_checksum,run_id FROM finding_charts where fc_name = "
                    + SqlUtils.sqlEscapedString(fcList[i].getFcName())
                    + " AND finding_charts.run_id = " + runId;
            stdlog_.debug(sqlText);
            selStmt = conn.createStatement();
            rs = selStmt.executeQuery(sqlText);
            boolean fileNameExists = false;
            fcId = -1;
            fcName = null;
            fcChecksum = -1;
            fcRunId = -1;
            while (rs.next()) {
                fileNameExists = true;
                fcId = rs.getInt("fc_id");
                fcName = rs.getString("fc_name").trim();
                fcChecksum = rs.getLong("fc_checksum");
                fcRunId = rs.getLong("run_id");
            }
            rs.close();
            selStmt.close();
            if (fileNameExists) {
                if (fcList[i].getFcName().trim().equals(fcName)
                    && fcList[i].getCheckSum() != fcChecksum) {
                    Vector vec = getOBIds(conn, fcId);
                    if (vec.size() > 0) {
                        StringBuffer buf = messageSameName(fcName, vec);
                        throw new SQLException(buf.toString());
                    }
                    updateFindingChartContents(conn, fcId, fcList[i]);
                }
                updateFinds(conn, boId, fcId, observatory);
                continue;
            }
            //if (!fileNameExists && !fileContentsExists) {

            byte[] fcData = fcList[i].getFcData();
            ByteArrayInputStream instream = new ByteArrayInputStream(fcData);
            fcId = DbaseIOHandler.getDbId();
            sqlText =
                "INSERT INTO finding_charts"
                    + " (fc_id,run_id,fc_name,fc_type,fc_checksum,fc_data,observatory,last_mod_date)"
                    + " VALUES (?,?,?,?,?,?,?,?)";
            stdlog_.debug(sqlText);
            PreparedStatement stmt = conn.prepareStatement(sqlText);
            stmt.setLong(1, fcId);
            stmt.setLong(2, runId);
            stmt.setString(3, fcList[i].getFcName());
            stmt.setString(4, fcList[i].getFcType());
            stmt.setLong(5, checksum);
            stmt.setBinaryStream(6, instream, (int) instream.available());
            stmt.setString(7, observatory);
            stmt.setLong(8, Convert.getCurrentTime());
            stmt.execute();
            stmt.close();
            updateFinds(conn, boId, fcId, observatory);
        }
    }

    /**
     * Return which observatory a run will be executed at.
     * @param conn
     * @param runId
     * @param sqlText
     * @return String
     * @throws SQLException
     */

    public static String getObservatory(Connection conn, long runId)
        throws SQLException {
        ResultSet rs;
        String sql = "SELECT observatory from " 
        + opcDB_ + "..opc_telescopes opc_telescopes, " 
        + opcDB_ + "..sched_rep sched_rep " 
        + " where sched_rep.run_id = "
        + runId + " AND sched_rep.tel = opc_telescopes.code";
        stdlog_.debug(sql);
        Statement observatoryStmt = conn.createStatement();
        rs = observatoryStmt.executeQuery(sql);
        String observatory = null;
        while (rs.next()) {
            observatory = rs.getString("observatory");
        }
        rs.close();
        observatoryStmt.close();
        return observatory;
    }

    /**
     * Error Message when same name has been discovered
     * @param fcName
     * @param vec
     * @return StringBuffer
     */

    public static StringBuffer messageSameName(String fcName, Vector vec) {
        StringBuffer buf = new StringBuffer();
        buf.append("You have already checked in a finding chart\n");
        buf.append("which has the same name but different contents");
        buf.append("(" + fcName);
        buf.append(")\n");
        buf.append(
            "If you wish to checkin your ob choose one of the following options\n");
        buf.append(
            "1. Rename the finiding chart to a different name than ("
                + fcName
                + ")\n");
        buf.append(
            "2. Checkout all of the OB's that are using this finding chart\n");
        buf.append(
            "   Attach the new finding chart and checkin. See list below\n");
        for (int j = 0; j < vec.size(); j++) {
            buf.append((String) vec.elementAt(j).toString() + "\n");
        }
        return buf;
    }

    /**
     * Error Message when same contents have been discovered.
     * @param i
     * @param fcName
     * @param vec
     * @return StringBuffer
     */

    private static StringBuffer sameContentsMessage(
        int i,
        String fcName,
        Vector vec) {
        StringBuffer buf = new StringBuffer();
        buf.append("You have already checked in a finding chart\n");
        buf.append("which has a different name but the same contents");
        buf.append("(" + fcName);
        buf.append(")\n");
        buf.append(
            "If you wish to checkin your ob choose one of the following options\n");
        buf.append("1. Change the finding chart contents\n");
        buf.append("2. Re-attach the orignal finding chart (" + fcName + ")\n");

        buf.append(
            "3. Checkout all of the OB's that are using this finding chart\n");
        buf.append(
            "   Attach the new finding chart and checkin. See list below :\n");
        for (int j = 0; j < vec.size(); j++) {
            if (i == 0)
                buf.append("   ");
            buf.append((String) vec.elementAt(j).toString() + "\n");
        }
        return buf;
    }

    /**
     * Error Message when same name have been discovered.
     * @param fcName
     */

    private static StringBuffer sameNameMessage(String fcName) {
        StringBuffer buf = new StringBuffer();
        buf.append("You have already checked in a finding chart\n");
        buf.append("which has the same name");
        buf.append("(" + fcName);
        buf.append(") but different contents\n");
        return buf;
    }

    /**
     * Error Message when same name have been discovered.
     * @param fcName
     */

    private static StringBuffer sameChecksumMessage(long checksum) {
        StringBuffer buf = new StringBuffer();
        buf.append("You have already checked in a finding chart\n");
        buf.append("which has the same checksum");
        buf.append("(" + checksum);
        buf.append(")\n");
        return buf;
    }

    /**
     * Insert into the finds table fc_id and the ob_id. The finds 
     * table has a one to many relationship to finding charts.
     * @param conn
     * @param boId
     * @param fcId
     * @throws SQLException
     */

    public static void updateFinds(
        Connection conn,
        long boId,
        long fcId,
        String observatory)
        throws SQLException {
        String sqlText;
        sqlText =
            "INSERT INTO finds"
                + " (fc_id,ob_id,observatory)"
                + " VALUES (?,?,?)";
        stdlog_.debug(sqlText);
        PreparedStatement stmt = conn.prepareStatement(sqlText);
        stmt.setLong(1, fcId);
        stmt.setLong(2, boId);
        stmt.setString(3,observatory);
        stmt.execute();
        stmt.close();
    }

    /**
     * Update the finding chart contents when there are no other OB's in the
     * database pointing to a given finding chart.
     * @param conn
     * @param fc_id
     * @param name
     * @throws ObjectIOException
     * @throws SQLException
     */

    public static void updateFindingChartContents(
        Connection conn,
        long fcId,
        FindingChart fcChart)
        throws ObjectIOException, SQLException {
        String updText =
            "UPDATE finding_charts set fc_data = ?,fc_checksum = ?,last_mod_date = ? where fc_id = ?";
        stdlog_.debug(updText);

        ByteArrayInputStream instream =
            new ByteArrayInputStream(fcChart.getFcData());
        PreparedStatement stmt = conn.prepareStatement(updText);
        stmt.setBinaryStream(1, instream, (int) instream.available());
        stmt.setLong(2, fcChart.getCheckSum());
        stmt.setLong(3, Convert.getCurrentTime());
        stmt.setLong(4, fcId);
        stmt.execute();
        stmt.close();
    }

    /**
     * Update the finding chart name when there are no other OB's
     * in the database pointing to a given finding chart.
     * @param conn
     * @param fc_id
     * @param name
     * @throws ObjectIOException
     * @throws SQLException
     */

    public static void updateFindingChartName(
        Connection conn,
        long fc_id,
        String name)
        throws ObjectIOException, SQLException {
        String sqlText;
        long fcId;
        String updText =
            "UPDATE finding_charts set fc_name =  ?,last_mod_date = ? where fc_id  = ?";
        stdlog_.debug(updText);
        PreparedStatement stmt = conn.prepareStatement(updText);
        stmt.setString(1, name);
        stmt.setLong(2, Convert.getCurrentTime());
        stmt.setLong(3, fc_id);
        stmt.executeUpdate();
        stmt.close();
    }

    /**
     * Update the finding chart notes given an fcId.
     * @param fc_id
     * @param notes
     * @throws ObjectIOException
     * @throws SQLException
     */

    public static void updateFindingChartNotes(long fcCheckSum, String notes)
        throws ObjectIOException {
        Config cfg = Config.getCfg();
        String updText =
            "UPDATE "
                + obrepDbName
                + "..finding_charts"
                + " set "
                + obrepDbName
                + "..finding_charts.fc_notes = ?"
                + " where "
                + obrepDbName
                + "..finding_charts.fc_checksum = ?";
        Connection conn = null;
        PreparedStatement stmt = null;

        SQLException error = null;
        stdlog_.debug("FC ID " + fcCheckSum);
        stdlog_.debug("Notes " + notes);
        try {
            conn =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());
            stdlog_.debug(updText);
            stmt = conn.prepareStatement(updText);
            stmt.setString(1, notes);
            stmt.setLong(2, fcCheckSum);
            stmt.executeUpdate();
            stmt.close();
        } catch (SQLException e) {
            error = e;
        }
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException e) {
                if (e == null) {
                    error = e;
                }
            }
        }
        if (conn != null) {
            ConnectionPool.getPool().releaseConnection(conn);
        }
        if (error != null) {
            throw new ObjectIOException(error);
        }
    }

    private static Vector getOBIds(Connection conn, long fcId)
        throws SQLException {
        String sqlText;
        Statement selStmt;
        ResultSet rs;
        sqlText = "SELECT DISTINCT ob_id from finds where fc_id = " + fcId;
        stdlog_.debug("Get OB Ids Text " + sqlText);
        selStmt = conn.createStatement();
        rs = selStmt.executeQuery(sqlText);
        Vector vec = new Vector();
        while (rs.next()) {
            int obId = rs.getInt("ob_id");
            if (!vec.contains(new Integer(obId))) {
                vec.addElement(new Integer(obId));
            }
        }
        return vec;
    }

    /**
     * Check if the finding chart exists for a given finding chart name.
     * @param conn
     * @param name
     * @return boolean
     * @throws SQLException
     */

    private static boolean findingChartNameExists(Connection conn, String name,long runId)
        throws SQLException {
        String sqlText;
        Statement selStmt;
        ResultSet rs;
        sqlText =
            "SELECT fc_name from finding_charts,finds where fc_name = "
                + SqlUtils.sqlEscapedString(name)
                + " AND finding_charts.fc_id = finds.fc_id"
                + " AND finding_charts.run_id = " + runId;
        stdlog_.debug("Finding Chart Name Exists " + sqlText);
        selStmt = conn.createStatement();
        rs = selStmt.executeQuery(sqlText);
        boolean exists = false;
        while (rs.next()) {
            exists = true;
        }
        return exists;
    }

    /**
     * Check if the finding chart checksum already exists
     * @param conn
     * @param name
     * @return boolean
     * @throws SQLException
     */

    private static boolean findingChartContentsExists(
        Connection conn,
        long checksum)
        throws SQLException {
        String sqlText;
        Statement selStmt;
        ResultSet rs;
        sqlText =
            "SELECT fc_checksum from finding_charts,finds where fc_checksum = "
                + checksum
                + " AND finding_charts.fc_id = finds.fc_id";

        stdlog_.debug("Finding Chart Contents Exists " + sqlText);

        selStmt = conn.createStatement();
        rs = selStmt.executeQuery(sqlText);
        boolean exists = false;
        while (rs.next()) {
            exists = true;
        }
        return exists;
    }

    /**
     * Delete all entries in the finds tables which links 
     * an OB to a finding_charts table. Once finding charts are 
     * entered into the finding_charts database table they cannot be removed
     * Those finding_charts that do not have entries in the finds table will be
     * removed when the observing run associated will be marked complete
     * 
     * @param stmt
     * @param id
     * @throws SQLException
     */

    public static void delete(Statement stmt, long id) throws SQLException {
        String sqlDel = "delete from finds where ob_id = " + id;
        stdlog_.debug(sqlDel);
        stmt.executeUpdate(sqlDel);
    }

    /**
     * Simply calls the delete and wrtie  operation.
     * @param conn
     * @param boId
     * @param runId
     * @param fcs
     * @throws SQLException
     * @throws ObjectIOException
     */

    public static void update(
        Connection conn,
        long boId,
        long runId,
        FindingChart[] fcs)
        throws SQLException, ObjectIOException {
        Statement delStmt = conn.createStatement();
        delete(delStmt, boId);
        write(conn, boId, runId, fcs);

    }

}
