package org.eso.ohs.phase2.apps.ot.configreport;

import java.util.Vector;

/**
 * A parameter in a config report definition. The parameter has a name and
 * a set of OB parameter names that are used to extract the value from the OB
 * when generating a report.
 *
 * As order of OB parameters is important in config reports, the parameters
 * must be added to the ConfigReportParameter object in the order required.
 *
 * @author Nick Kornweibel
 */

public class ConfigReportParameter {
    
    /**
     * The string token used in an OB Parameter to seperate it into
     * a template parameter name and a Paf Parameter name.
     */
    protected static final String PafIndirectionToken = "->";

    protected String name_ = "";
    protected Vector parameters_ = new Vector();

    public ConfigReportParameter(String name) {
        name_ = name;
    }

    /**
     * @return the name of this config report parameters.
     */
    public String getName() {
        return name_;
    }

    /**
     * Add a OB parameter to this config report parameter. Does nothing if
     * the param is null or empty.
     */
    public void addParameter(String param) {
        if (param != null && param.length() > 0) {
            parameters_.addElement(param);
        }
    }

    /**
     * @return the number of OB parameters in this config report parameter.
     */
    public int getParameterCount() {
        return parameters_.size();
    }

    /**
     * @return the full parameter String at the index given.
     */
    public String getParameter(int i) {
        return (String)parameters_.elementAt(i);
    }

    /**
     * @return true if the request OB parameter addresses as Paf sub-parameter.
     * @see ConfigReport
     */
    public boolean isPafParameter(int i) {
        String param = getParameter(i);
        return param.indexOf(PafIndirectionToken) > 0;
    }

    /**
     * @return String - the OB parameter at the specified index.
     * (With out the Paf redirection parameter, if present). 
     */
    public String getTemplateParameter(int i) {
        String param = getParameter(i);
        int pafIdx = param.indexOf(PafIndirectionToken);
        if (pafIdx >= 0) {
            param = param.substring(0, pafIdx);
        }
        return param;
    }

    /**
     * @return String - the PAF parameter at the specified index. 
     * (With out the Template parameter, if present).
     */
    public String getPafParameter(int i) {
        String param = getParameter(i);
        int pafIdx = param.indexOf(PafIndirectionToken);
        if (pafIdx >= 0) {
            param = param.substring(pafIdx+(PafIndirectionToken.length()));
        } else {
            param = null;
        }
        return param;
    }
}
