package org.eso.ohs.phase2.apps.ot.configreport;

import java.util.Vector;

/**
 * A record of data in a Config Report.
 * Each record stores:
 * - the set of values for the parameters in the config report (in values_)
 * - the set of OBs that match this config report:
 *      - the ob ids in obIds_
 *      - the ob program names in progNames_
 *      - the ob names in obNames_
 *
 * @author Nick Kornweibel
 */

public class ConfigReportRecord {
    
    protected ConfigReport parentReport_;
    protected Vector values_ = new Vector();
    protected Vector obIds_ = new Vector();
    protected Vector obTypes_ = new Vector();
    protected Vector obStatus_ = new Vector();
    protected Vector progNames_ = new Vector();
    protected Vector obNames_ = new Vector();
    protected Vector raVals_ = new Vector();

    public ConfigReportRecord(ConfigReport r) {
        parentReport_ = r;
    }

    /**
     * @return the report that created this ConfigReportRecord.
     */
    public ConfigReport getReport() {
        return parentReport_;
    }

    /**
     * @return the values for the config report - the actual parameter
     * names used to extract the report can be gotten from the ConfigReport
     * using getReport().getConfig().
     */
    public Vector getReportValues() {
        return values_;
    }

    /**
     * @return Vector of Strings of ob id of the OBs that matched the report.
     */
    public Vector getOBIds() {
        return obIds_;
    }

    /**
     * @return Vector of Strings of types for the OBs.
     */
    public Vector getOBTypes() {
        return obTypes_;
    }

    /**
     * @return Vector of Strings of types for the OBs.
     */
    public Vector getOBStatus() {
        return obStatus_;
    }


    /**
     * @return Vector of Strings of program names of OBs that match
     * this config report.
     */
    public Vector getOBProgramNames() {
        return progNames_;
    }

    /**
     * @return Vector of Strings of ob names of OB that matched this report.
     */
    public Vector getOBNames() {
        return obNames_;
    }

    /**
     * @return Vector of Strings of RA values of the OBs that match this
     * report.
     */
    public Vector getRAs() {
        return raVals_;
    }
}
