package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.table.TableColumn;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbExecutionTime;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DisplayTableAsTextAction;
import org.eso.ohs.core.dbb.client.MoonDistance;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminated;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminatedListener;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;

/**
 * The QueueItemsBView class is a DB search engine used for displaying queues of
 * OBs according to a user-definable set of properties
 * 
 * @version $Revision: 1.35 $
 * @author Gorgio Passera DMD/ESO <gpassera@eso.org> <br>
 *         ChangeLog: cmavros 01/07/02 - Add local method addDefaultColumns()
 */
public class ExecutionSequenceBView extends OTQueueBrowserView {

   
    private static final long serialVersionUID = 1L;

    public static final String cvsID_ = "$Id: ExecutionSequenceBView.java,v 1.35 2005/11/29 10:54:49 ddorigo Exp $";

    /** Standard log channel */
    private static Logger stdlog_ = Logger
            .getLogger(ExecutionSequenceBView.class);

    private JButton moveToTopButton;

    private JButton removeAllButton;

    private JButton displayTextButton;

    private static ExecutionSequenceBView execSequence_ = new ExecutionSequenceBView(
            new Queue(), new OBTreeView());

  
   
   
    private String hostName = null;

    /**
     * Default constructor <br>
     * ChangeLog: Gorgio Passera <DMD/ESO> <gpassera@eso.org>
     * 
     */
    private ExecutionSequenceBView(Queue newQueue, OBTreeView obTreeView) {
        super();
        initBaseClass(newQueue, "", false, false, false, obTreeView);
        setSize(new Dimension(650, 600));
        buildToolBar();
        layoutBrowser();
        view_.addQueryTerminatedListener(new QueryTerminatedListener());

        // view_.addQueryTerminatedListener(listener_);
    }

    public void addNotify() {
        super.addNotify();
    }

    public void layoutBrowser() {
        // obTreeView_ = new OBTreeView();

        // stdlog_.debug("Column Count " + table.getColumnCount());
        JPanel content = new JPanel();
        content.setLayout(new BorderLayout());
        /* JSplitPane */
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, view_,
                tree_);

        // JSplitPane splitPane2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
        // splitPane,executionSequenceView);
        splitPane.setDividerLocation(300);

        content.add(splitPane, BorderLayout.CENTER);
        content.add(toolBar_, BorderLayout.SOUTH);
        setLayout(new BorderLayout());
        add(content, BorderLayout.CENTER);
        add(new JLabel(title_), BorderLayout.NORTH);
    }

    public static ExecutionSequenceBView getInstance() {
        return execSequence_;
    }

    public void obPropertyChange(OBEvent ev) {
        if (ev.getValue().equals("Delete")) {
            long[] obIds = ev.getOBsId();
            if (obIds != null && obIds.length > 0)
                removeOBs(obIds);
        } else {
            long[] OBIDs = ev.getOBsId();
            for (int i = 0; i < OBIDs.length; i++) {
                if (containsOB(OBIDs[i])) {
                    refreshViewsStatus(OBIDs[i], ev.getValue());
                }
            }
        }
    }

    public void removeOBs(long[] selectedOBIDs) {
        // try {
        // QueueManager.deleteOBFromOTExecSequence(selectedOBIDs);
        int[] rowIDs = getRowIDs(selectedOBIDs);
        removeRows(rowIDs);
        setCurrentSelection();
        // } catch (ObjectIOException e) {
        // ErrorMessages.announceIOError(view_.getTable().getTable(), e);
        // }

    }

    protected void buildToolBar() {
        createActions();
        toolBar_ = new JToolBar();
        toolBar_.setFloatable(false);
        removeAllButton = new JButton("Remove All");
        toolBar_.add(removeAllButton);

        removeAllButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                try {
                    QueueManager.deleteOBFromOTExecSequence(getAllOBsId(false));
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(view_.getTable().getTable(),
                            e);
                }
                // removeOBs();
            }

        });

        removeButton = new JButton("Remove");
        removeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                try {
                    QueueManager
                            .deleteOBFromOTExecSequence(getSelectedOBIDs(false));
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(view_.getTable().getTable(),
                            e);
                }
                // removeOBs();
            }

        });

        toolBar_.add(removeButton);

        moveUpButton = new JButton("Move Up");
        moveUpButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    moveUpRows();
                    QueueManager.updateExecOBsSequences(getAllOBsId(false));
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(view_.getTable().getTable(),
                            e);
                }
            }
        });

        moveDownButton = new JButton("Move Down");
        moveDownButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    moveDownRows();
                    QueueManager.updateExecOBsSequences(getAllOBsId(false));
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(view_.getTable().getTable(),
                            e);
                }
            }

        });

        moveToTopButton = new JButton("Move To Top");
        moveToTopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    moveToTop();
                    QueueManager.updateExecOBsSequences(getAllOBsId(false));
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(view_.getTable().getTable(),
                            e);
                }
            }
        });

        displayTextButton = new JButton();

        toolBar_.add(moveUpButton);
        toolBar_.add(moveDownButton);
        toolBar_.add(moveToTopButton);
        toolBar_.add(displayAction);
        toolBar_.add(obHistoryAction);
        toolBar_.add(findingChartAction);
        toolBar_.add(displayTextButton);
        setEnabled(false);
    }

    public void setDisplayTextAction(JFrame frame) {
        displayTextButton.setAction(new DisplayTableAsTextAction(view_,
                "Execution Sequence", "Execution Sequence", frame));
        displayTextButton.setText("Text View");
    }

    public void setEnabled(boolean enabled) {
        removeAllButton.setEnabled(enabled);
        removeButton.setEnabled(enabled);
        moveUpButton.setEnabled(enabled);
        moveDownButton.setEnabled(enabled);
        moveToTopButton.setEnabled(enabled);
        displayAction.setEnabled(enabled);
        obHistoryAction.setEnabled(enabled);
        findingChartAction.setEnabled(enabled);
        displayTextButton.setEnabled(enabled);
    }

    /**
     * Return a set of OBsID that belong to the execution sequence
     * 
     * @param maxNumOBs
     *            specifies the number of ObsId requested
     * @return a set of OBsID that belong to the execution sequence or null
     */
    public long[] getOBsIdFromExecutionSequence(int maxNumOBs) {
        try {

            if (maxNumOBs > getView().getRows()) {
                maxNumOBs = getView().getRows();
            }
            long[] returnList = new long[maxNumOBs];
            for (int i = 0; i < maxNumOBs; i++) {
                returnList[i] = getOBID(i);
            }
            return returnList;
        } catch (NumberFormatException e) {
            stdlog_.error(e);
            return null;
        }
    }

    public boolean isOBSelected() {
        int[] selectedRows = getSelectedRows();
        return (selectedRows.length != 0);
    }

    /**
     * Build DB browser view: SELECT panel
     * 
     * <br>
     * ChangeLog: Gorgio Passera
     * 
     * @exception SQLException
     *                JDBC exception occurs
     * @exception RemoteException
     *                RMI exception occurs
     * @exception MalformedURLException
     *                malformed url i.e. servername//port
     * @exception NotBoundException
     *                client could not bind to server
     */
    public void buildColumns()
            throws NotBoundException, MalformedURLException, SQLException,
            RemoteException {
        String label_name;
        String column_name;
        column_name = "ob_id";
        label_name = "OB ID";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(oes_table, column_name,
                label_name));
        addDefaultColumns();
        DbbPanel panel = new DbbPanel("Selected Columns", 9);
        view_.setColumnsPanel(panel, "North", false);
    }

    /** Build DB browser view: rows selection panel */
    public void buildRows() {

        DbbPanel panel = new DbbPanel("Selection Criteria", 5);
        DbbSqlChunk c;

        String label_name;
        String column_name;

        DbbSqlOperator op_equal = null;

        try {
            op_equal = new DbbSqlOperator("=");
        } catch (Exception e) {
            stdlog_.error(e);
        }

        // Queue ID
        /*
         * label_name = "Queue ID"; column_name = "sch_id";
         * 
         * c = new DbbSqlChunk( es_table, column_name, label_name, op_equal,
         * Integer.class); queueIDTF = new DbbTextField(label_name, c);
         * panel.addWidget(queueIDTF);
         * 
         * queueIDTF.set(new
         * Long(Config.getCfg().uniqueToTableId(queue.getId())));
         * queueIDTF.setEnabled(false);
         */

        label_name = "hostname";
        column_name = "hostname";
        c = new DbbSqlChunk(oes_table, column_name, label_name, op_equal,
                String.class);
        DbbTextField hostTextField = new DbbTextField(label_name, c);
        panel.addWidget(hostTextField);
        hostTextField.set(getHostName());
        hostTextField.setEnabled(false);
        view_.setWherePanel(panel, "North", false);
    }

    /**
     * Method to append OBs that pass verification test to the Execution
     * Sequence View. Called by endOfAction method in
     * ot.actions.ExecutionSequenceAction
     * 
     * @param setOBIDs
     *            list of OBs to be added to the Execution Sequence View <br>
     *            ChangeLog: cmavros 12/03/02 1709/OT
     */
    public boolean appendOBs(long[] appendIds) {
        JTable table = view_.getTable().getTable();
        stdlog_.debug("Column Count " + table.getColumnCount());
        String hostName = getHostName();
        try {
            QueueManager.appendOBsToExecSequence(appendIds, view_.getRows(),
                    hostName);
            queue.appendQueueItems(appendIds, getTypes(appendIds));
            setOrderDirtyFlag(false);
            startQuery();
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(table, e);
        }
        return true;
    }

    /**
     * @param hostName
     * @return String
     */
    private String getHostName() {
        if (hostName == null) {
            try {
                hostName = InetAddress.getLocalHost().getHostName().trim();
                stdlog_.debug("IP hostname: " + hostName);
            } catch (UnknownHostException e) {
                JTable table1 = view_.getTable().getTable();
                String[] text = new String[1];
                text[0] = "Cannot resolve IP host name.";
                ErrorMessages.announceError(table1, text, e);
            }
        }
        return hostName;
    }

    /**
     * Method to add default columns to the Execution browser
     * 
     * @exception SQLException
     *                JDBC exception occurs
     * @exception RemoteException
     *                RMI exception occurs
     * @exception MalformedURLException
     *                malformed url i.e. servername//port
     * @exception NotBoundException
     *                client could not bind to server <br>
     *                ChangeLog: cmavros 01/07/02 Bug 1807/OT
     */
    public void addDefaultColumns()
            throws NotBoundException, MalformedURLException, SQLException,
            RemoteException {
        String label_name;
        String column_name;

        // Type
        column_name = "type";
        label_name = "Type";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob_table, column_name,
                label_name));

        // Status
        column_name = "status";
        label_name = "Status";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob_table, column_name,
                label_name));

        // OB Name
        label_name = "OB Name";
        column_name = "item_name";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob_table, column_name,
                label_name));

        label_name = "Instrument";
        column_name = "instrument";

        engine_.addDefaultColumnsChunk(new DbbSqlChunk(od_table, column_name,
                label_name));

        // ProgID
        label_name = "ProgID";
        column_name = "progid";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(sr_table, column_name,
                label_name, String.class));

        // PI (surname)
        label_name = "PI";
        column_name = "surname_dl";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(eu_table, column_name,
                label_name));

        // RA
        label_name = "RA";
        column_name = "ra";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(tp_table, column_name,
                label_name, DbaseRA.class));

        // Dec
        label_name = "Dec";
        column_name = "dec";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(tp_table, column_name,
                label_name, DbaseDec.class));

        // Exec Time
        label_name = "ExecTime";
        column_name = "execution_time";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob_table, column_name,
                label_name, DbbExecutionTime.class));

        // Rank Class
        label_name = "RankClass";
        column_name = "rank_class";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(sr_table, column_name,
                label_name, String.class));

        // UsrP
        label_name = "UsrP";
        column_name = "userPriority";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob_table, column_name,
                label_name));

        // Seeing
        label_name = "Seeing";
        column_name = "seeing";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cs_table, column_name,
                label_name));

        // Airmass
        label_name = "Airmass";
        column_name = "airmass";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cs_table, column_name,
                label_name));

        // Sky Tran
        label_name = "SkyTran";
        column_name = "sky_transparency";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cs_table, column_name,
                label_name));

        // FLI
        label_name = "FLI";
        column_name = "fli";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cs_table, column_name,
                label_name));

        // MoonDis
        label_name = "MoonDis";
        column_name = "moon_distance";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cs_table, column_name,
                label_name, MoonDistance.class));

        // Strehl
        label_name = "Strehl";
        column_name = "strehl_ratio";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cs_table, column_name,
                label_name));

        // label_name = "Finding Charts";
        // column_name = "fc_name";
        // engine_.addDefaultColumnsChunk(
        // new DbbSqlChunk(finds_view, column_name, label_name));

        // label_name = "hostname";
        // column_name = "hostname";
        // engine_.addDefaultColumnsChunk(
        // new DbbSqlChunk(oes_table, column_name, label_name));

    }

    private class QueryTerminatedListener implements DbbQueryTerminatedListener {
        public void queryTerminatedAction(DbbQueryTerminated ev) {
            // now we have a full table we can remove the typ
            // column from the view
            JTable table = view_.getTable().getTable();
            TableColumn typeColumn = table.getColumn("Type");
            table.removeColumn(typeColumn);
        }

    }

    /** Build DB browser view: sort selection panel */
    public void buildSort() {

        DbbPanel panel = new DbbPanel("Sorting Criteria", 1);
        DbbSqlChunk c;
        DbbArrowButton w;
        String column_name;
        String label_name;

        // sequence
        label_name = "sequence";
        column_name = "sequence";

        c = new DbbSqlChunk(oes_table, column_name, label_name);
        w = new DbbArrowButton(label_name, c, 1);

        panel.addWidget(w);
        w.actionPerformed(new ActionEvent(this, 0, null));

        view_.setSortPanel(panel, "West", false);
    }

    protected void refreshViewsStatus(long obId, String status) {
        int[] rows = getRowsForOBID(obId);
        for (int i = 0; i < rows.length; i++) {
            view_.setValueAt(status, rows[i], OTDbbView.STATUS);
        }
        if (rows.length > 0)
            currentRow_ = rows[0];
        setCurrentSelection();
    }
}
