package org.eso.ohs.phase2.apps.ot.wizard;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.eso.ohs.core.gui.widgets.OHSWizard;
import org.eso.ohs.core.gui.widgets.OHSWizardPage;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * This wizard page allows the user to compute and show a timeline
 * based on the current list of visible OBs
 * 
 * @version $Revision: 1.1 $
 * @author amchavan & pnunes, Oct 17, 2005
 */

// $Id: TimelinePage.java,v 1.1 2005/10/17 15:33:44 amchavan Exp $

public class TimelinePage extends OHSWizardPage {
    JLabel labelTitle = new JLabel();

    JLabel labelPageTitle = new JLabel();

    JLabel labelAlgorithm = new JLabel();

    JComboBox cbTimelineAlgorithm = new JComboBox();
    
    JLabel labelInfo = new JLabel();

    private OrangWidgetsData dataWidgets;

    /**
     * Default constructor
     * @param defaultData 
     */
    public TimelinePage( OHSWizard controller, OrangWidgetsData defaultData ) {
        super( controller );
        dataWidgets = defaultData;
        initializePanel();
    }

    /**
     * Adds fill components to empty cells in the first row and first column of
     * the grid. This ensures that the grid spacing will be the same as shown in
     * the designer.
     * 
     * @param cols
     *            an array of column indices in the first row where fill
     *            components should be added.
     * @param rows
     *            an array of row indices in the first column where fill
     *            components should be added.
     */
    void addFillComponents(Container panel, int[] cols, int[] rows) {
        Dimension filler = new Dimension(10, 10);

        boolean filled_cell_11 = false;
        CellConstraints cc = new CellConstraints();
        if (cols.length > 0 && rows.length > 0) {
            if (cols[0] == 1 && rows[0] == 1) {
                /** add a rigid area */
                panel.add(Box.createRigidArea(filler), cc.xy(1, 1));
                filled_cell_11 = true;
            }
        }

        for (int index = 0; index < cols.length; index++) {
            if (cols[index] == 1 && filled_cell_11) {
                continue;
            }
            panel.add(Box.createRigidArea(filler), cc.xy(cols[index], 1));
        }

        for (int index = 0; index < rows.length; index++) {
            if (rows[index] == 1 && filled_cell_11) {
                continue;
            }
            panel.add(Box.createRigidArea(filler), cc.xy(1, rows[index]));
        }

    }


    public JPanel createPanel() {
        JPanel jpanel1 = new JPanel();
        FormLayout formlayout1 = new FormLayout(
                "FILL:DEFAULT:NONE,LEFT:DEFAULT:NONE,FILL:DEFAULT:GROW(1.0),FILL:78PX:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:41PX:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE",
                "FILL:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:GROW(1.0),CENTER:DEFAULT:NONE" );
        CellConstraints cc = new CellConstraints();
        jpanel1.setLayout( formlayout1 );

        labelPageTitle.setFont( new Font( "sansserif", Font.ITALIC, 11 ) );
        labelPageTitle.setName( "labelPageTitle" );
        labelPageTitle.setText( "Step 5: choose timeline algorithm" );
        jpanel1.add( labelPageTitle, cc.xywh( 2, 4, 4, 1 ) );

        labelAlgorithm.setName( "labelAlgorithm" );
        labelAlgorithm.setText( "Algorithm" );
        jpanel1.add( labelAlgorithm, cc.xy( 2, 7 ) );

        cbTimelineAlgorithm.setName( OrangWizard.TIMELINE_ALGORITHM );
        jpanel1.add( cbTimelineAlgorithm, cc.xy( 3, 7 ) );

        labelInfo.setName( "labelInfo" );
        labelInfo.setText( "0 Observation blocks loaded" );
        jpanel1.add( labelInfo, cc.xy( 2, 18 ) );

        jpanel1.add( WizardTitlePanel.createPanel(), new CellConstraints( 1, 1, 11, 1,
                CellConstraints.FILL, CellConstraints.DEFAULT ) );
        addFillComponents( 
                    jpanel1,
                    new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 },
                    new int[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, 16, 17, 18 } );
        return jpanel1;
    }


    protected void initializePanel() {
        setLayout( new BorderLayout() );
        add( createPanel(), BorderLayout.CENTER );

        // We hard-code here the list of algorithms
        // (for the time being...)
        String[] algoritmList = new String[1];
        algoritmList[0] = "Early-greedy";
        for( int i = 0; i < algoritmList.length; i++ ) {
            cbTimelineAlgorithm.addItem( algoritmList[i] );
        }
        listenTo( cbTimelineAlgorithm );
    }

    protected boolean isReady() { 
        return true;
    }
    
    public void setInfo(String info) {
    	this.labelInfo.setText(info);
    }
}
