package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Category;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ReadmeAttributesFactory;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
import org.eso.ohs.phase2.apps.p2pp.ReadmeEditView;
import org.jdom.JDOMException;

/**
 * Action class to check-out the selected Readme.
 * 
 * <p>
 * <br>
 * 
 * @author Bob Kemp <br>
 *         ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP
 *         AppServer. <br>
 *         ChangeLog: cmavros 28JUNE02 Correct the validation of OBs through
 *         P2PP AppServer.
 */
public class CheckOutReadmeAction extends ActionSuperclass {

    private static Category stdlog_ = Category.getInstance(CheckInAction.class);

    /** FolderGridView for the folder containing the selected OB */
    private HierarchyView hView_;

    private String actionName_ = null;

    /**
     * Constructs a check-in action.
     */
    public CheckOutReadmeAction(HierarchyView hView) {
        super((JComponent)hView,"CheckOut Readme");
        hView_ = hView;
    }

    public void actionPerformedImpl(ActionEvent ev) {
        Persistence objMgr = Persistence.getInstance();
        CalibrationBlock cb = null;
        ObservingRun or=null;
        if(!OHSApplication.ensureAuthenticated())
             return;
        try {
            or = hView_.getSelectedFolder().findObsRun();
        } catch (ObjectIOException e1) {
            ErrorMessages.announceIOError(hView_,e1);
        }
        if (!or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_STANDARD)) {
            JOptionPane.showMessageDialog(hView_.getTopLevelAncestor(),
                    new Object[] {
                            "Only Readme(s) for service-mode Observing Runs",
                            "can be checked-out." }, "Cannot Check-Out",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        int option = JOptionPane
                .showConfirmDialog(
                        hView_.getTopLevelAncestor(),
                        new Object[] { "Are you sure you want to check-out the readme file for this run ?" },
                        "Continue with Check-Out?", JOptionPane.OK_CANCEL_OPTION);
        switch (option) {
        case JOptionPane.CLOSED_OPTION:
            return;
        case JOptionPane.CANCEL_OPTION:
            return;
        }
        Instrument inst = InstrumentList.getInstance().getInstrument(or.getInstCode(),or.getIPVersion());
        if(!inst.hasReadmeData()) {
                JOptionPane.showMessageDialog(
                    hView_,
                    new Object[] {
                        "There is no readme configuration available for this observing run.",
                        "Try download refresh observing runs",
                        "If this still does not work then contact usg-help@eso.org",
                        },
                    "Readme Available",
                    JOptionPane.ERROR_MESSAGE);
                return;
        }

        
        Persistence persistence = Persistence.getInstance();
        try {
            String xmlString = persistence.checkOutReadmeFile(or);
            stdlog_.debug("XML String " + xmlString);
            Readme readme = or.getReadme();
            if(readme == null) {
                // Create the readme files again.
                readme = new Readme(or.getIPVersion());                
                try {
                    readme.setRedmeAttributes(ReadmeAttributesFactory.creteReadmeAttribFromXMLString(inst,xmlString));
                } catch (JDOMException e2) {
                    ErrorMessages.announceIOError(hView_,e2);
                    e2.printStackTrace();
                }
            } else {
                try {
                    readme.setRedmeAttributes(ReadmeAttributesFactory.refreshReadmeAttribFromXMLString(readme.getRedmeAttributes(),xmlString));
                } catch (JDOMException e2) {
                    ErrorMessages.announceIOError(hView_,e2);
                    e2.printStackTrace();
                }                
            }
            readme.setCheckedIn(false);
            MainWindow.getPrincipalWindow().setReadmeIcon(true);

            // force the firing of an event
            or.setReadme(readme);
            if(ReadmeEditView.getInstance().isVisible())
                 ReadmeEditView.getInstance().init(or);
            
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(hView_,e);
        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceNoObject(hView_,e);
        } catch (IOException e) {
            ErrorMessages.announceIOError(hView_,e);
        }
    }
}