package org.eso.ohs.phase2.orang;

import org.eso.ohs.dfs.ObTargetConstraintInfo;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.RunProperties;
import org.eso.ohs.dfs.SchedRun;
import org.eso.ohs.dfs.TimeInterval;

/**
 * Class ObDataContainer
 * 
 * Contains OB information, time intervals
 * 
 * @version $Revision: 1.8 $
 * @author ddorigo, Aug 10, 2005
 */

// $Id: ObDataContainer.java,v 1.8 2005/11/15 13:45:40 ddorigo Exp $
public class ObDataContainer {

    private ObTargetConstraintInfo obInfo_ = null;

    private TimeInterval [] timeIntervals_ = null;
    private TimeInterval [] siderealTimeIntervals_ = null;
    
    private SchedRun schedRun_ = null;

    private RunProperties runProp = null;
 

    /**
     * Creates a new object getting an   ObTargetConstraintInfo as parameter
     * @param obInfo ObTargetConstraintInfo loaded from the database
     */
    public ObDataContainer(ObTargetConstraintInfo obInfo) {
        obInfo_ = obInfo;
        
    }            

    /**
     * Creates a new object getting an  ObservationBlock as parameter 
     * @param ob ObservationBlock object loaded from the database
     */
    public ObDataContainer(ObservationBlock ob) {
		
    	ObTargetConstraintInfo info = new ObTargetConstraintInfo();
    	info.setAirmass(ob.getConstraintSet().getAirmass());
    	info.setCloneParentId(ob.getCloneParentId());
    	info.setCreationDate(ob.getCreationDate());
    	info.setDbaseId(ob.getDbaseId());
    	info.setDec(ob.getTarget().getDeclination());
    	info.setDiffDec(ob.getTarget().getDiffDec());
    	info.setDiffRA(ob.getTarget().getDiffRA());
    	info.setExecutionTime(ob.getExecutionTime());
    	info.setFractionalLunarIllumination(ob.getConstraintSet().getFractionalLunarIllumination());
    	info.setId(ob.getId());
    	info.setIsNew(ob.isNew());
    	info.setLastModifiedDate(ob.getLastModifiedDate());
    	info.setLineNumber(ob.getLineNumber());
    	info.setMoonAngularDistance(ob.getConstraintSet().getMoonAngularDistance());
    	info.setName(ob.getName());
    	info.setOwnerId(ob.getOwnerId());
    	info.setPropDec(ob.getTarget().getProperDeclination());
    	info.setRa(ob.getTarget().getRA());
    	info.setRunId((int)ob.getObsRun().getRunId());
    	info.setSeeing(ob.getConstraintSet().getSeeing());
    	info.setSkyTransparency(ob.getConstraintSet().getSkyTransparency());
    	info.setStatus(ob.getStatus());
    	info.setStrehlRatio(ob.getConstraintSet().getStrehlRatio());
    	info.setType(ob.getType());
    	info.setUserPriority(ob.getUserPriority());
    	obInfo_ = info;
    	
    	schedRun_ = new SchedRun();    	
    	schedRun_.setRankClass(ob.getObsRun().getRankClass());
    	
    	
    	
    	timeIntervals_ = ob.getTimeIntervals();
    	siderealTimeIntervals_ = ob.getSTTimeIntervals();
    	
	}
    
    
    /**
	 * Creates an array of ObDataContainer object out of an array of ObservationBlock objects
	 * 
	 * @param obs
	 * @return
	 */
	public static  ObDataContainer [] createContainers( ObservationBlock [] obs) {
		 ObDataContainer [] res = new  ObDataContainer [obs.length];
		 
		 for (int i = 0; i < res.length; i++) {
			res[i] = new ObDataContainer(obs[i]);
			
		}
		 
		 return(res);
	}

	/**
	 * @return ObTargetConstraintInfo
	 */
	public ObTargetConstraintInfo getObInfo() {
        return obInfo_;
    };

    /**
     * @return TimeInterval[]
     */
    public TimeInterval[] getTimeIntervals() {
    	if(timeIntervals_ == null)
    		return new TimeInterval[0];
    	return timeIntervals_;
    }
    
	/**
	 * @return TimeInterval[]
	 */
	public TimeInterval[] getSiderealTimeIntervals() {		
		if(siderealTimeIntervals_ == null)
    		return new TimeInterval[0];
    	return siderealTimeIntervals_;
	}
    

	public SchedRun getSchedRun() {
		return schedRun_;
	}
	public void setSchedRun(SchedRun schedRun) {
		this.schedRun_ = schedRun;
	}
	

	/** 
	 * Adds a TimeInterval object to the sidereal time intervals
	 * @param ti TimeInterval
	 */
	public void addSiderealTimeInterval(TimeInterval ti) {
		
		if (siderealTimeIntervals_ != null ) {
			TimeInterval [] newTi = new TimeInterval[siderealTimeIntervals_.length+1];		
			for (int i = 0; i < siderealTimeIntervals_.length; i++) {
				newTi[i] = siderealTimeIntervals_[i];
			}
			newTi[newTi.length-1] = ti;
			siderealTimeIntervals_ = newTi;
		
		} else {
			siderealTimeIntervals_ = new TimeInterval[1];
			siderealTimeIntervals_[0] = ti;
		}
		
		
	}
	
	/** 
	 * Adds a TimeInterval object to the  time intervals
	 * @param ti TimeInterval
	 */
	public void addTimeInterval(TimeInterval ti) {
		
		if (timeIntervals_ != null ) {
			TimeInterval [] newTi = new TimeInterval[timeIntervals_.length+1];		
			for (int i = 0; i < timeIntervals_.length; i++) {
				newTi[i] = timeIntervals_[i];
			}
			newTi[newTi.length-1] = ti;
			timeIntervals_ = newTi;
			
		} else {
			timeIntervals_ = new TimeInterval[1];
			timeIntervals_[0] = ti;
		}
		
	}
	
	
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf
                .append(obInfo_.getRunId() + " (OB Id=" + obInfo_.getDbaseId()
                        + ") ");
        buf.append("\n");
        buf
                .append(" RA: ")
                .append(obInfo_.getRa())
                .append(", ")
                .append("Dec: ")
                .append(obInfo_.getDec());
                
        
        ObservationBlock.fillTimeInterval(buf, timeIntervals_);
        ObservationBlock.fillTimeInterval(buf, siderealTimeIntervals_);
        
        if ( schedRun_ != null ) {
        	
        	buf
        	.append("Run ID: ")
        	.append(schedRun_.getRunId());
        } else {
        	buf.append("Run ID: null ");
        }
        buf.append("\n");
        	
        
        return buf.toString();
    }

    public void setRunProperties(RunProperties runProp) {
       this.runProp = runProp;
    }

    public RunProperties getRunProperties() {
        return runProp;
    } 
    
    public long getObId() {
        return obInfo_.getDbaseId();
    }
    
    /**
     * The equals checks if an object has the same obId properties
     * 
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if (obj!= null&& obj instanceof ObDataContainer) {
           ObDataContainer currentOb = (ObDataContainer) obj;
           result = (this.getObId() == currentOb.getObId());
        }
        return result;
        } 
   
}