package org.eso.ohs.phase2.orang;

import java.util.Vector;

import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.Seeing;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.dfs.StrehlRatio;
import org.eso.ohs.dfs.WeatherConstraint;

public class ObWeatherFilter {

    private static final double TOLERANCE = 0.0001;

    private ObDataContainer[] obDataArray_;

    public ObWeatherFilter(ObDataContainer[] obList) {
        setObDataArray(obList);
    }

    public ObWeatherFilter() {
        obDataArray_ = new ObDataContainer[0];
    }

    public void setObDataArray(ObDataContainer[] obList) {
        obDataArray_ = obList;
    }

    public ObDataContainer[] filterObArray(ConstraintSet weatherConditions) {
        Seeing seeing = new Seeing(weatherConditions.getSeeing());
        SkyTransparency sky = new SkyTransparency(weatherConditions
                .getCurrDbSkyTransparency());
        StrehlRatio ratio = new StrehlRatio(weatherConditions.getStrehlRatio());
        WeatherConstraint currentConstraint = new WeatherConstraint(seeing,
                sky, ratio);
        ObDataContainer[] obArray = filterObArray(currentConstraint);

        return obArray;
    }

    public ObDataContainer[] filterObArray(WeatherConstraint currentConstraint) {
        Vector obDataVector = new Vector();

        for (int i = 0; i < obDataArray_.length; i++) {
            ObDataContainer currentOb = obDataArray_[i];

            boolean seeingOk = true;
            if (currentConstraint.isSeeingEnable()) {
                if ((currentConstraint.getSeeing().doubleValue() - currentOb
                        .getObInfo()
                        .getSeeing()) < TOLERANCE) {
                    seeingOk = true;
                } else {
                    seeingOk = false;
                }
            } else {
                seeingOk = true;
            }

            boolean skyOk = true;
            if (currentConstraint.isSkyEnable()) {
                String skyString = currentOb.getObInfo().getSkyTransparency();
              //  String currentSkyDB = SkyTransparency.toDbSkyTransparency(skyString);
              
                if (currentConstraint.getSky().get().compareTo(skyString) <= 0) {
                    skyOk = true;
                } else {
                    skyOk = false;
                }
            } else {
                skyOk = true;
            }
            boolean strehlRatioOk = true;
            if (currentConstraint.isStrehlRatioEnable()) {
                if (Math.abs(currentOb.getObInfo().getStrehlRatio()) < TOLERANCE) {
                    strehlRatioOk = true;
                } else {
                    // the ratio has to be considered
                    if (currentOb.getObInfo().getStrehlRatio()
                            - currentConstraint.getStrehlRatio().get() < TOLERANCE) {
                        strehlRatioOk = true;
                    } else {
                        strehlRatioOk = false;
                    }
                }
            } else {
                strehlRatioOk = true;
            }
            if (strehlRatioOk == true && skyOk == true && seeingOk == true) {
                obDataVector.addElement(currentOb);
            }
        }
        return (ObDataContainer[]) obDataVector
                .toArray(new ObDataContainer[obDataVector.size()]);
    }
}
