/*
 * Created by amchavan on Aug 30, 2005
 */
package org.eso.ohs.phase2.timeline;

import java.util.Date;

import org.eso.ohs.phase2.visibility.PCF;

/**
 * This interface defines a task that can be placed on a timeline.
 * 
 * @version $Revision: 1.4 $
 * @author amchavan, Aug 30, 2005
 */

// $Id: TimelineTask.java,v 1.4 2005/10/20 15:32:53 amchavan Exp $

public interface TimelineTask {

    /** @return The amount of time it takes to complete
     *          this task, in seconds */
    public long getDuration();
    
    /** @return The task's unique ID */
    public long getID();
    
    /** @return The PCF describing the task's preferred start
     *          times. Higher values indicate better start times.
     *          Zero-valued intervals indicate unfeasible times. */
    public PCF getPreference();
    
    /** 
     * Compares two tasks by priority. 
     * The result is a negative integer if this TimelineTask object
     * has higher priority (is "more important") than the
     * argument task. 
     * The result is a positive integer if this TimelineTask object
     * has lower priority than the argument task.
     * The result is zero if the tasks have equal priority.
     * 
     * @param anotherTask  The task to be compared.
     * 
     * @return  0 if the argument task has same priority as 
     *          this task; a value less than 0 if this task has
     *          higher priority than the task argument; and a 
     *          value greater than 0 if this task has lower
     *          priority than the task argument.
     */
    public int compareTo( TimelineTask anotherTask );
    
    /** @return The tasks's scheduled start time, or <code>null</code>
     *          if the task was not scheduled.
     */
    public Date getStartTime();
    
    /** @return The tasks's scheduled end time, or <code>null</code>
     *          if the task was not scheduled.
     */
    public Date getEndTime();
    
    /** 
     * @param start This tasks's scheduled start time, or <code>null</code>
     *              if the task was not scheduled.
     */
    public void setStartTime( Date start );
    
    /** @return The task's name or description, will be used for reports */
    public String getName();
    
    /** @return <code>true</code> if this task can be executed,
     *          <code>false</code> otherwise
     */
    public boolean isDoable();
}
