package org.eso.ohs.scripting;

/**
 * ConstraintSetAdaptor is an adaptor class that takes a ConstraintSet
 * object and exposes the ConstraintSetAPI interface.
 *
 * @author Nick Kornweibel
 */

import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.persistence.Config;

public class ConstraintSetAdaptor implements ConstraintSetAPI {

    private ConstraintSet cSet_ = null;

    /**
     * Constructor accepts a valid ConstraintSet object
     * @param the constraint set
     */

    public ConstraintSetAdaptor(ConstraintSet conSet) {
        cSet_ = conSet;
    }


    /**
     * Get the value of the Airmass as a string
     * @return the air mass
     */

    public String getAirmass() {
        return ""+cSet_.getAirmass();
    }

    /**
     * Get value for Fractional Lunar Illumination as string
     * @return the Fractional Lunar Illumination
     */

    public String getFractionalLunarIllumination() {
        return ""+cSet_.getFractionalLunarIllumination();
    }

    /**
     * Get value for Sky Transparency
     * @return the Sky Transparency
     */

    public String getSkyTransparency() {
        return ScriptingUtilities.nullToEmptyString(cSet_.getSkyTransparency());
    }

    /**
     * Get value for Moon Angular Distance
     * @return the Moon Angular Distance
     */

    public int getMoonAngularDistance() {
        return cSet_.getMoonAngularDistance();
    }

    /**
     * Get value for Seeing as a string
     * @return the seeing
     */

    public String getSeeing() {
        return ""+cSet_.getSeeing();
    }

    /**
     * Get value for Tel Temp
     * @return the Tel Temp
     */

    public int getTelTemp() {
        return cSet_.getTelTemp();
    }

    /**
     * Get value for Water Vapour
     * @return the water vapou
     */

    public String getWaterVapour() {
        return ScriptingUtilities.nullToEmptyString(cSet_.getWaterVapour());
    }

    /**
     * Get value for Strehl Ratio
     * @return the strehl ratio
     */

    public float getStrehlRatio() {
        return cSet_.getStrehlRatio();
    }
    
    
    /**
     * Get the baseline for the 
     * @return the baseline
     */

    public String getBaseline() {
        return cSet_.getBaseline();
    }


    /**
     * Get value for Description property
     * @return the description
     */

    public String getDescription() {
        return ScriptingUtilities.nullToEmptyString(cSet_.getDescription());
    }

    /**
     * Get the name of the Constraint set
     * @return the name of the constraint set
     */

    public String getName() {
        return ConstraintSet.getDisplayName();
    }

    /**
     * Get the Id of the Constraint set
     * @return the id of the Constraint set (long value in String)
     */

    public String getId() {
        String rc;
        if (cSet_.getDbaseId() != 0) {
            rc = ""+Config.getCfg().uniqueToTableId(cSet_.getDbaseId());
        } else {
            rc = ""+Config.getCfg().uniqueToTableId(cSet_.getId());
        }
        return rc;
    }
}
