package org.eso.ohs.scripting;

/**
 * This is an adaptor class accepting an array of TimeInterval objects and
 * presenting a TimeIntervalSetAPI
 *
 * @author Nick Kornweibel
 */

import org.eso.ohs.dfs.TimeInterval;

public class TimeIntervalSetAdaptor implements TimeIntervalSetAPI {

    private TimeInterval[] timeIntervals_;

    /**
     * Constructor - accepts an array of TimeInterval objects
     * @param array of TimeIntervals
     */

    public TimeIntervalSetAdaptor(TimeInterval[] timeInts) {
        timeIntervals_ = timeInts;
    }

    /**
     * Get the number of time intervals in the set
     * @return the number of time intervals
     */

    public int getNumberOfTimeIntervals() {
        return timeIntervals_.length;
    }

    /**
     * Get the time interval at the specified index, or null if
     * the index is invalid.
     * @param index of time interval (starting from 0)
     * @return the TimeInterval at the index
     */

    public TimeIntervalAPI getTimeInterval(int index) {
        TimeIntervalAdaptor rc;
        if (index >= 0 && index < timeIntervals_.length) {
            rc = new TimeIntervalAdaptor(timeIntervals_[index]);
        } else {
            rc = null;
        }
        return rc;
    }

    /**
     * Get the name of this time interval set
     * @return the name of this time interval set
     */

    public String getName() {
        return "TimeIntervalSet";
    }

    /**
     * Get the id for this time interval set - this returns simply
     * the name of the time interval set, not a unique id
     * @return string containing the id for this time interval set
     */

    public String getId() {
        return getName();
    }
}
